<?php

/**
 * Fonctions pour l'import ENT 
 * @Projet SambaEdu
 * @Auteurs Equipe Sambaedu
 * @Licence Distribue sous la licence GPL
 * @Repertoire: includes/
 * file: ent.inc.php
 * 
 * Le csv format utilisé correspond à celui utilisé par l'ENT Openent : 
 * première ligne avec les champs : 
 * "externalId";"Id Siecle";"type";"lastname";"firstName";"originalLogin";"login","code";"functions";"structures";"allClasses";"childrens";"parents";"birthDate";"sex";"allGroups"
 *  id : identifiant unique ENT 
 * Id Siecle : numero eleve siecle (eleves uniquement)
 * + Nom : nom en majuscule => TEST
 * + Prénom : prénom => Elève
 * + Type : Student|Teacher|Personnel|Relative|Guest
 * originalLogin : login d'origine, prenom.nom(x) x étant un nombre pour les doublons.
 * login : login simplifié chois par l'utilisateur => repris en priorité par Sambaedu
 * code : mot de passe initial => repris par sambaedu si il existe
 * functions : Personnel administatif => array()
 * structures : nom de(s) l'établissemnt(s) en majuscules ,séparés par des virgules => array()
 * + allClasses : classe(s), séparées par des virgules => array()
 * allGroups : groupe(s), séparés par des virgules
 * birthDate : date de naissance AAAAMMJJ
 * sex : sexe M/F
 * 
 * seuls les champs avec + sont obligatoires
 * 
 * Des mappages sont possibles pour d'autres types d'imports, voir ci-dessous.
 * La même structure est possible en json ou xml 
 */
function map_ent_fields($keys, &$type = "")
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    if (empty($type))
        $type = "csv";
    $map['csv_openent'] = array(
        "Id" => "externalId",
        "Id Siecle" => "Id Siecle",
        "Type" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "Login" => "originalLogin",
        "Alias de login" => "login",
        "Code d'activation" => "code",
        "Fonction(s)" => "functions",
        "Structure(s)" => "structures",
        "Classe(s)" => "allClasses",
        "Enfant(s)" => "childrens",
        "Parent(s)" => "parents"
    );
    $map['csv_pronote'] = array(
        "NOM" => "lastName",
        "PRENOM" => "firstName",
        "CLASSES" => "allClasses",
        "GROUPES" => "allGroups",
        "FONCTION" => "type",
        "EMAIL" => "email",
        "CIVILITE" => "sex",
        "DATE_NAISS" => "birthDate",
        "SEXE" => "sex"
    );

    $map['csv_pronote_eleve'] = array(
        "NOM" => "lastName",
        "PRENOM" => "firstName",
        "GROUPES" => "allGroups",
        "EMAIL" => "email",
        "CIVILITE" => "sex",
        "DATE_NAISS" => "birthDate",
        "N NATIONAL" => "Id Siecle",
        "CLASSE" => "allClasses",
        "SEXE" => "sex"
    );

    $map['kosmos'] = array(
        "Identifiant ENT" => "externalId",
        "sexe" => "sex",
        "année de naissance" => "birthDate",
        "Profil" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "classe" => "allClasses"
    );
    $map['csv'] = array(
        "sexe" => "sex",
        "année de naissance" => "birthDate",
        "Profil" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "classe" => "allClasses"
    );

    $ret = array();
    foreach ($keys as $index => $key) {
        // détection d'un fichier pronote eleve
        if ($type == "csv_pronote" && $key == "CLASSE") {
            $type = "csv_pronote_eleve";
        }
    }
    foreach ($keys as $index => $key) {
        if (isset($map[$type][$key])) {
            $ret[$index] = $map[$type][$key];
        } else {
            $ret[$index] = $key;
        }
    }
    return $ret;
}

function map_profil($profil, $type = "")
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    if (empty($type))
        $type = "csv";
    $map['csv_openent'] = array(
        "Elève" => "Student",
        "Enseignant" => "Teacher",
        "Personnel" => "Personnel",
        "Invité" => "Guest",
        "Parent" => "Relative"
    );
    $map['kosmos'] = array(
        "Eleve" => "Student",
        "Professeur" => "Teacher",
        "Personnel" => "Personnel",
        "Parent" => "Relative",
        "Autre" => "Personnel"
    );
    $map['csv'] = array(
        "Eleve" => "Student",
        "Professeur" => "Teacher",
        "Personnel" => "Personnel"
    );
    $map['csv_pronote'] = array(
        "ENS" => "Teacher"
    );
    $map['manuel'] = array(
        "Eleves" => "Student",
        "Profs" => "Teacher",
        "Administratifs" => "Personnel"
    );
    if (isset($map[$type][$profil])) {
        $ret = $map[$type][$profil];
    } else {
        $ret = $profil;
    }
    return $ret;
}

function map_classe($classes, $type = "")
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    $ret = array();
    if (empty($type)) {
        $type = "csv";
    }
    if ($type == "csv_openent" || $type == "csv" || $type == "csv_pronote" || $type == "csv_pronote_eleve") {
        if (count($classes) > 0) {
            foreach ($classes as $classe) {
                $ret[]['name'] = trim($classe);
            }
        }
        if ($type == "csv_pronote_eleve") {
            $ret = [
                $ret[0]
            ];
        }
    } elseif ($type == "kosmos") {
        if (count($classes) > 0) {
            foreach ($classes as $classe) {
                $classe = explode("~", $classe);
                if (isset($classe[1]))
                    $ret[]['name'] = $classe[1];
            }
        }
    }
    return $ret;
}

function map_birthdate($date, $type = "")
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    $ret = array();
    if (empty($type)) {
        $type = "csv";
    }
    switch ($type) {
        case "csv_pronote":
        case "csv_pronote_eleve":
            $ddn = explode("/", $date);
            return implode("", array_reverse($ddn));
            break;
        default:
            return preg_replace("/-/", "", $date);
    }
}

function map_sex($sex)
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    switch ($sex) {
        case "Mme":
        case "Mlle":
        case "F":
            $sex = "F";
            break;
        default:
            $sex = "M";
    }
    return $sex;
}

function cmp_classe($a, $b)
{
    $ca = count($a['allClasses']);
    $cb = count($b['allClasses']);
    if ($ca > $cb)
        return - 1;
    elseif ($cb > $ca)
        return 1;
    else
        return 0;
}

function remplace_separateurs($chaine)
{
    $retour = strtr($chaine, array(
        "'" => ' ',
        '-' => ' ',
        '_' => ' '
    ));

    return $retour;
}

/**
 * * simplifie le nom
 * - enleve l'apostrophe, les tirets...
 * - colle les particules
 * -
 *
 * @param string $name
 * @return string
 */
function simplify_name(string $name, $short = false)
{
    $name = strtolower(remplace_accents($name));
    $name = remplace_separateurs($name);
    $length = strlen($name);
    $mots = explode(" ", $name);

    if (strlen($mots[0]) < 3 && ! empty($mots[1])) {
        // on colle la particule (l el le al n mc....)
        $mots[1] = $mots[0] . $mots[1];
        array_shift($mots);
        $length --;
    }
    foreach ($mots as $n => $mot) {
        if (empty($mot)) {
            unset($mots[$n]);
        }
    }
    if ($length < 12) {
        if ($short) {
            $name = substr(implode("", $mots), 0, 12);
        } else {
            $name = substr(implode("-", $mots), 0, 12);
        }
    } else {
        $name = substr(array_shift($mots), 0, 12);
    }
    return $name;
}

function simplify_firstname(string $name, $short = false)
{
    $name = strtolower(remplace_accents($name));
    $name = remplace_separateurs($name);
    $length = strlen($name);
    $mots = explode(" ", $name);

    if (strlen($mots[0]) < 3 && ! empty($mots[1])) {
        // on colle la particule (l el le al n mc....)
        $mots[1] = $mots[0] . $mots[1];
        array_shift($mots);
        $length --;
    }
    foreach ($mots as $n => $mot) {
        if (empty($mot)) {
            unset($mots[$n]);
        }
    }
    if ($short) {
        $name = "";
        foreach ($mots as $mot) {
            $name .= substr($mot, 0, 1);
        }
    } else {
        if ($length < 12) {
            $name = substr(implode("-", $mots), 0, 12);
        } else {
            $name = substr(array_shift($mots), 0, 12);
        }
    }
    return $name;
}

function doublon_cn($cn)
{
    $ret = [
        'cn' => $cn,
        'nb' => ""
    ];
    if (preg_match("/^([a-z-.]*)([0-9]*)/", $cn, $m)) {
        $ret['cn'] = $m[1];
        $ret['nb'] = $m[2];
    }
    return $ret;
}

function mk_cn($config, $nom, $prenom, $cnPolicy = "", $nb = 0)
{
    /*
     * # Valeurs de l'cnPolicy
     * # 0: prenom.nom
     * # 1: prenom.nom tronque a 19
     * # 2: p.nom tronque a 15
     * # 3: pnom tronque a 8
     * # 4: nomp tronque a 8
     * # 5: nomprenom tronque a 18
     */
    if (empty($cnPolicy))
        $cnPolicy = $config['cnPolicy'];

    switch ($cnPolicy) {
        case 0:
        case 1:
            $nom = simplify_name($nom);
            $prenom = simplify_firstname($prenom);
            if ($nb > 0) {
                $nom .= $nb;
            }
            if (strlen($nom) + strlen($prenom) > 18) {
                $prenom = substr($prenom, 0, 18 - strlen($nom));
            }
            $cn = $prenom . "." . $nom;
            break;
        case 2:
            $nom = simplify_name($nom);
            $prenom = simplify_firstname($prenom, true);

            if ($nb > 0) {
                $nom .= $nb;
            }
            $cn = $prenom . "." . $nom;
            break;
        case 3:
            $nom = simplify_name($nom, true);
            $prenom = simplify_firstname($prenom, true);

            if ($nb == 0) {
                $debut_nom = substr($nom, 0, 8 - strlen($prenom));
            } else {
                $debut_nom = substr($nom, 0, (8 - strlen($prenom) - strlen($nb))) . $nb;
            }
            $cn = $prenom . $debut_nom;
            break;
        case 4:
            $nom = simplify_name($nom, true);
            $prenom = simplify_firstname($prenom, true);

            if ($nb == 0) {
                $debut_nom = substr($nom, 0, 8 - strlen($prenom));
            } else {
                $debut_nom = substr($nom, 0, (8 - strlen($prenom) - strlen($nb)));
            }
            $cn = $debut_nom . $prenom;
            if ($nb > 0) {
                $cn .= $nb;
            }
            break;
        case 5:
            $nom = simplify_name($nom);
            $prenom = simplify_firstname($prenom);

            if ($nb > 0) {
                if (strlen($nom) + strlen($prenom) + strlen($nb) > 17) {
                    $prenom = substr($prenom, 0, 17 - strlen($nom) - strlen($nb));
                }
                $cn = $nom . $prenom . $nb;
            } else {
                if (strlen($nom) + strlen($prenom) > 17) {
                    $prenom = substr($prenom, 0, 17 - strlen($nom));
                }
                $cn = $nom . $prenom;
            }
            break;
        default:
            $cn = false;
    }

    // Pour faire disparaitre les caracteres speciaux restants:
    $cn = strtolower(preg_replace("/[^a-z0-9.-]/", "", $cn));

    // Pour eviter les _ en fin d'UID... pb avec des connexions machine de M$7
    $cn = preg_replace("/_*$/", "", $cn);
    return $cn;
}

function oauth_login($config)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'clientId' => $config['openent_oauth2_id'], // The client ID assigned to you by the provider
        'clientSecret' => $config['openent_oauth2_pass'], // The client password assigned to you by the provider
        'redirectUri' => $config['openent_oauth2_redirect_uri'],
        'urlAuthorize' => $config['openent_uri'] . $config['openent_oauth2_auth'],
        'urlAccessToken' => $config['openent_uri'] . $config['openent_oauth2_token'],
        'urlResourceOwnerDetails' => $config['openent_uri'] . $config['openent_oauth2_userinfo'],
        'scopes' => [
            'userinfo'
        ]
    ];
    set_oauth_proxy($config, $opt);
    $provider = new \League\OAuth2\Client\Provider\GenericProvider($opt);
    // Fetch the authorization URL from the provider; this returns the
    // urlAuthorize option and generates and applies any necessary parameters
    // (e.g. state).
    try {
        $authorizationUrl = $provider->getAuthorizationUrl();
        $accessToken = $provider->getAccessToken('authorization_code', [
            'code' => $_GET['code']
        ]);

        // Get the state generated for you and store it to the session.
        $_SESSION['oauth2state'] = $provider->getState();

        return $authorizationUrl;
    } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {

        // Failed to get the access token or user details.
        return "erreur de connexion Oauth2 : " . $e->getMessage();
    }
}

function oauth_callback($config)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'clientId' => $config['openent_oauth2_id'], // The client ID assigned to you by the provider
        'clientSecret' => $config['openent_oauth2_pass'], // The client password assigned to you by the provider
        'redirectUri' => $config['openent_oauth2_redirect_uri'],
        'urlAuthorize' => $config['openent_uri'] . $config['openent_oauth2_auth'],
        'urlAccessToken' => $config['openent_uri'] . $config['openent_oauth2_token'],
        'urlResourceOwnerDetails' => $config['openent_uri'] . $config['openent_oauth2_userinfo'],
        'scopes' => [
            'userinfo'
        ]
    ];
    set_oauth_proxy($config, $opt);

    $provider = new \League\OAuth2\Client\Provider\GenericProvider($opt);

    // If we don't have an authorization code then get one

    if (! isset($_GET['code'])) {

        // Fetch the authorization URL from the provider; this returns the
        // urlAuthorize option and generates and applies any necessary parameters
        // (e.g. state).
        $authorizationUrl = $provider->getAuthorizationUrl();

        // Get the state generated for you and store it to the session.
        $_SESSION['oauth2state'] = $provider->getState();

        // Redirect the user to the authorization URL.
        header('Location: ' . $authorizationUrl);
        exit();

        // Check given state against previously stored one to mitigate CSRF attack
    } elseif (empty($_GET['state']) || (isset($_SESSION['oauth2state']) && $_GET['state'] !== $_SESSION['oauth2state'])) {

        echo "state : " . $_GET['state'] . "<br>";
        echo "<pre>" . print_r($_SESSION, true) . "</pre>";
        unset($_SESSION['oauth2state']);
        exit('Invalid state');
    } else {

        try {
            $time = time();
            $existingAccessToken = apcu_fetch($_GET['state']);

            // $existingAccessToken = $_SESSION['accesstoken'] ?? false;
            if (! empty($existingAccessToken) && $existingAccessToken->hasExpired()) {
                $accessToken = $provider->getAccessToken('refresh_token', [
                    'refresh_token' => $existingAccessToken->getRefreshToken()
                ]);
            } else {
                // Try to get an access token using the authorization code grant.
                $accessToken = $provider->getAccessToken('authorization_code', [
                    'code' => $_GET['code']
                ]);
            }
            apcu_store($_GET['state'], $accessToken);
            $time = time() - $time;
            $message = "getAccessToken: " . $time . "s\n";
            error_log($message, 3, "/tmp/profil.log");
            // We have an access token, which we may use in authenticated
            // requests against the service provider's API.
            // echo $accessToken->getToken() . "\n";
            // echo $accessToken->getRefreshToken() . "\n";$global = $_POST['global'] ?? false;

            // echo $accessToken->getExpires() . "\n";
            // echo ($accessToken->hasExpired() ? 'expired' : 'not expired') . "\n";

            // Using the access token, we may look up details about the
            // resource owner.
            $time = time();
            $resourceOwner = $provider->getResourceOwner($accessToken);
            $userinfo = $resourceOwner->toArray();
            $time = time() - $time;
            $message = "getResourceOwner: " . $time . "s\n";
            error_log($message, 3, "/tmp/profil.log");
            $time = time();
            // le login peut-il être différent de celui de l'ent ?
            // OUI ! incohérence entre userinfo, et directory, qui ne renvoient pas la même valeur. Du coup on est obligé d'aller chercgher dans l'ad
            $filter = "(&(objectclass=user)(|(title=" . $userinfo['externalId'] . ")(cn=" . $userinfo['login'] . ")))";
            $user = search_ad($config, $filter, "filter", $config['dn']['people']);
            $time = time() - $time;
            $message = "search_ad: " . $time . "s\n";
            error_log($message, 3, "/tmp/profil.log");

            $_SESSION['login_ent'] = $userinfo['login'];
            // $_SESSION['login'] = $userinfo['login'];
            $_SESSION['login'] = $user[0]['cn'];
            $_SESSION['auth'] = "ent";
            // }
            $_SESSION['accesstoken'] = $accessToken;
            $_SESSION['hasPw'] = $userinfo['hasPw'] ?? NULL;
            ;
            $_SESSION['forceChangePassword'] = $userinfo['forceChangePassword'] ?? NULL;
            // header("Location:" . $config['se4_url']);
            header("Location:../index.html");
        } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {

            // Failed to get the access token or user details.
            exit("erreur de connexion Oauth2 : " . $e->getMessage());
        }
    }
}

/**
 * teste la connexion de l'utilisateur; renvoie les donées ou false.
 *
 * @param unknown $config
 * @param unknown $login
 * @param unknown $passwd
 * @return boolean|mixed
 */
function check_ent_passwd($config, $login, $passwd)
{
    if ($config['ent_type'] == "openent") {
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['openent_uri'],
            'cookies' => true
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('POST', 'auth/login', [
                'form_params' => [
                    'email' => $login,
                    'password' => $passwd
                ],
                'connect_timeout' => 10
            ]);
            if ($response) {
                $response = $client->request('GET', 'userbook/api/person');
                (string) $data = $response->getBody();
                $ent = json_decode($data, true);
                if ($ent['status'] != "ok") {
                    return false;
                }
            } else {
                $ent = false;
            }
        } catch (\GuzzleHttp\Exception\RequestException $e) {
            $ent = false;
        }
        return $ent;
    }
    return true;
}

/**
 * change le password ent de l'utilisateur
 *
 * cette fonction suppose que le mdp d'origine est connu.
 * en cas de perte l'utilisateur doit untiliser la fonction mot de passe perdu de l'ent.
 *
 * @param unknown $config
 * @param unknown $login
 * @param unknown $passwd
 * @return boolean|mixed
 */
function change_ent_passwd($config, $login, $passwd, $newpasswd)
{
    if ($config['ent_type'] == "openent") {
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['openent_uri'],
            'cookies' => true
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('POST', 'auth/login', [
                'form_params' => [
                    'email' => $login,
                    'password' => $passwd
                ]
            ]);
            if ($response) {
                foreach ($client->getConfig('cookies')->toArray() as $cookie) {
                    if ($cookie['Name'] == "XSRF-TOKEN") {
                        $token = $cookie['Value'];
                    }
                }
                $response = $client->request('POST', 'auth/reset', [
                    'form_params' => [
                        'oldPassword' => $passwd,
                        'password' => $newpasswd,
                        'confirmPassword' => $newpasswd
                    ],
                    'headers' => [
                        'X-XSRF-TOKEN' => $token
                    ]
                ]);
                if ($response)
                    return true;
            }
        } catch (\GuzzleHttp\Exception\RequestException $e) {
            return false;
        }
    }
    return false;
}

/**
 * reinit le password ent de l'utilisateur
 *
 * cette fonction bascule lent en mode mdp perdu et renvoie le code d'activation
 * et le stocke dans apcu 'ent'
 *
 * @param unknown $config
 * @param unknown $originallogin
 * @param unknown $passwd
 * @return boolean|code
 */
function reinit_ent_passwd($config, $originallogin)
{
    sync_ent($config);
    if ($ent = apcu_fetch('ent')) {
        $key = array_search($originallogin, array_column($ent, 'originalLogin'));
        $code = $ent[$key]['code'];
        if ($code) {
            return $code;
        }

        if ($config['ent_type'] == "openent") {
            require_once (dirname(__FILE__) . '/../vendor/autoload.php');
            $opt = [
                'base_uri' => $config['openent_uri'],
                'cookies' => true
            ];
            curl_proxy_options($config, $opt);
            $client = new GuzzleHttp\Client($opt);
            try {
                $client->request('POST', 'auth/login', [
                    'form_params' => [
                        'email' => $config['openent_user'],
                        'password' => $config['openent_user_password']
                    ]
                ]);
            } catch (GuzzleHttp\Exception\ClientException $e) {
                $html .= "erreur lors de la connexion à l'ENT<br>\n";
                $html .= GuzzleHTTP\Psr7\str($e->getRequest());
                $html .= GuzzleHTTP\Psr7\str($e->getResponse());
                return false;
            }
            try {

                foreach ($client->getConfig('cookies')->toArray() as $cookie) {
                    if ($cookie['Name'] == "XSRF-TOKEN") {
                        $token = $cookie['Value'];
                    }
                }
                $response = $client->request('POST', 'auth/generatePasswordRenewalCode', [
                    'form_params' => [
                        'login' => $originallogin
                    ],
                    'headers' => [
                        'X-XSRF-TOKEN' => $token
                    ]
                ]);
                if ($response) {
                    (string) $data = $response->getBody();
                    $msg = json_decode($data, true);
                    $ent[$key]['code'] = $msg['renewalCode'];
                    apcu_store('ent', 600);
                    return ($msg['renewalCode']);
                }
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                return false;
            }
        }
    }
    return false;
}

/**
 * reinitialise le mot de passe dans l'ent et dans l'ad.
 * Si pas de methode de reinit dans l'ad, les stratégies de mot de passe s'appliquent
 *
 * @param unknown $config
 * @param array $user
 * @return string $html
 */
function reinit_passwd($config, array $user)
{
    $originallogin = explode("@", $user['userprincipalname'])[0] ?? $user['cn'];
    $pwdPolicy = $config['pwdPolicy'];
    $change = true;
    if ($userpwd = reinit_ent_passwd($config, $originallogin)) {
        $html = "Mot de passe provisoire de <b>" . $user['cn'] . "</b> réinitialisé par l'ENT : <b>" . $userpwd . "</b><br>";
    } else {
        switch ($pwdPolicy) {
            case 0: // date de naissance sans expiration (DANGER)
                $userpwd = $user['date'];
                $html .= gettext("Mot de passe de <b>" . $user['cn'] . "</b> réinitialisé à la date de naissance : <b>" . $userpwd . "</b><br>");
                if (! empty($config['no_passwd_change'])) {
                    $change = false;
                }
                break;
            case 2: // aleatoire
            case 3: // code initial, on met aléatoire
                $userpwd = create_random_password(8, true);
                $html .= gettext("Mot de passe de <b>" . $user['cn'] . "</b> r&#233;initialis&#233; &#224; : <b>" . $userpwd . "</b><br>");
                break;
            default: // code initial
                $userpwd = $config['activation_code'];
                $html .= gettext("Mot de passe de <b>" . $user['cn'] . "</b> r&#233;initialis&#233; &#224; : <b>" . $userpwd . "</b><br>");
        }
    }
    usersetpassword($config, $user['cn'], $userpwd, $change);
    return ($html);
}

/**
 * fixe l'alias de login dans l'ent; renvoie les données ou false.
 *
 * @param unknown $config
 * @param unknown $login
 * @param unknown $passwd
 * @return boolean|mixed
 */
function change_ent_alias($config, &$client, $ad_users, &$user, &$html, $manu = false)
{
    if ($config['ent_type'] == "openent" && ! $manu) {
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');

        if (empty($client)) {
            $opt = [
                'base_uri' => $config['openent_uri'],
                'cookies' => true
            ];
            curl_proxy_options($config, $opt);

            $client = new GuzzleHttp\Client($opt);
            try {
                $client->request('POST', 'auth/login', [
                    'form_params' => [
                        'email' => $config['openent_user'],
                        'password' => $config['openent_user_password']
                    ]
                ]);
            } catch (GuzzleHttp\Exception\ClientException $e) {
                $html .= "erreur lors de la connexion à l'ENT<br>\n";
                $html .= GuzzleHTTP\Psr7\str($e->getRequest());
                $html .= GuzzleHTTP\Psr7\str($e->getResponse());
                return false;
            }
        }
        $new = false;
        $increment = 0;
        $force_alias = $config['ent_force_alias'] ?? "0";
        while (! $new && $increment < 20) {
            if ($force_alias == 1) {
                $new_login = $user['ad']['cn'];
                $new = true;
            } elseif ($user['ent']['login'] != $user['ent']['originalLogin']) {
                $html .= " on garde l'alias ENT déjà en place. <br>";
                return true;
            } else {
                $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", $increment);
            }
            if ($new_login == $user['ent']['login']) {
                return true;
            } elseif ($new_login == $user['ent']['originalLogin']) {
                $html .= "Le login d'origine ENT est déjà  : " . $new_login . ", on efface l'alias.<br>";
                $new_login = "";
            }
            while (! empty($new_login) && in_array($new_login, array_column($ad_users, "cn")) && $force_alias == 0) {
                $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", ++ $increment);
            }
            if ($new_login == $user['ent']['login'] || $new_login == $user['ent']['originalLogin']) {
                $html .= "l'alias ENT est déjà  $new_login. Pas de changement d'alias <br>\n";
                return true;
            }
            try {
                foreach ($client->getConfig('cookies')->toArray() as $cookie) {
                    if ($cookie['Name'] == "XSRF-TOKEN") {
                        $token = $cookie['Value'];
                    }
                }

                $response = $client->request('PUT', 'directory/user/' . $user['ent']['id'], [
                    'json' => [
                        'loginAlias' => $new_login
                    ],
                    'headers' => [
                        'X-XSRF-TOKEN' => $token
                    ]
                ]);
                if ($response) {
                    (string) $data = $response->getBody();
                    $msg = json_decode($data, true);
                    if ($msg['id'] == $user['ent']['id']) {
                        $user['ent']['login'] = $new_login;
                        $html .= "Login ENT changé : " . $new_login . "<br>";
                        return true;
                    }
                }
            } catch (GuzzleHttp\Exception\ClientException $e) {
                $html .= "changement d'alias $new_login impossible,<br>\n";
                if (! empty($config['debug'])) {
                    $html .= GuzzleHTTP\Psr7\str($e->getRequest());
                    $html .= GuzzleHTTP\Psr7\str($e->getResponse()) . "<br>";
                }
                $increment ++;
            }
        }
        return false;
    } else {
        $new = false;
        $increment = 0;
        while (! $new && $increment < 20) {
            if ($user['ent']['login'] != $user['ent']['originalLogin']) {
                $html .= " on garde l'alias ENT déjà en place. <br>";
                return true;
            } else {
                $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", $increment);
            }
            if ($new_login == $user['ent']['login']) {
                return true;
            } elseif ($new_login == $user['ent']['originalLogin']) {
                $html .= "Le login d'origine ENT est déjà  : " . $new_login . ", on efface l'alias.<br>";
                $new_login = "";
            }
            while (! empty($new_login) && in_array($new_login, array_column($ad_users, "cn")) && $force_alias == 0) {
                $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", ++ $increment);
            }
            if ($new_login == $user['ent']['login'] || $new_login == $user['ent']['originalLogin']) {
                $html .= "l'alias ENT est déjà  $new_login. Pas de changement d'alias <br>\n";
                return true;
            } else {
            $user['ent']['login'] = $new_login;
            $html .= "Login ENT changé : " . $new_login . "<br>";
            return true;
            }
            $increment ++;
        }

        return false;
    }
}

/**
 * verrouille deverouille le compte si besoin
 *
 * @param
 *            $config
 * @param
 *            ent_user
 * @param
 *            block
 * @return etat
 */
function lock_ent_user($config, &$client, &$user, $block, &$html)
{
    if ($user['ent']['blocked'] != $block) {

        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        if (empty($client)) {
            $opt = [
                'base_uri' => $config['openent_uri'],
                'cookies' => true
            ];
            curl_proxy_options($config, $opt);

            $client = new GuzzleHttp\Client($opt);

            try {
                $client->request('POST', 'auth/login', [
                    'form_params' => [
                        'email' => $config['openent_user'],
                        'password' => $config['openent_user_password']
                    ]
                ]);
            } catch (GuzzleHttp\Exception\ClientException $e) {
                $html .= "erreur lors de la connexion à l'ENT<br>\n";
                $html .= GuzzleHTTP\Psr7\str($e->getRequest());
                $html .= GuzzleHTTP\Psr7\str($e->getResponse());
                return false;
            }
        }
        try {
            foreach ($client->getConfig('cookies')->toArray() as $cookie) {
                if ($cookie['Name'] == "XSRF-TOKEN") {
                    $token = $cookie['Value'];
                }
            }
            $response = $client->request('PUT', "auth/block/" . $user['ent']['id'], [
                'json' => [
                    'block' => $block
                ],
                'headers' => [
                    'X-XSRF-TOKEN' => $token
                ]
            ]);
            if ($response) {
                $user['ent']['blocked'] = $block;
                return true;
            } else {
                $html .= "erreur lors de l'operation $block sur " . $user['ent']['login'] . "<br>\n";
            }
        } catch (\GuzzleHttp\Exception\TransferException $e) {
            $html .= "erreur lors de l'operation $block sur " . $user['ent']['login'] . "<br>\n";
            $html .= GuzzleHTTP\Psr7\str($e->getRequest());
            $html .= GuzzleHTTP\Psr7\str($e->getResponse());
        }

        return false;
    } else {
        return true;
    }
}

/**
 * récupère la base utilisateurs dans l'ENT
 * les identifiants d'un administrateur doivent être saisis dans un ficiher de conf
 *
 * @param unknown $config
 * @return tableau des utilisateurs
 */
function sync_ent($config, $force = false)
{
    $ent = apcu_fetch('ent', $success);
    $html = "";
    if (@$config['ent_type'] == "openent") {
        if (! $success || $force) {
            require_once (dirname(__FILE__) . '/../vendor/autoload.php');
            $opt = [
                'base_uri' => $config['openent_uri'],
                'cookies' => true
            ];
            curl_proxy_options($config, $opt);
            $client = new GuzzleHttp\Client($opt);
            try {
                $client->request('POST', 'auth/login', [
                    'form_params' => [
                        'email' => $config['openent_user'],
                        'password' => $config['openent_user_password']
                    ]
                ]);
                $response = $client->request('GET', 'directory/export/users');
                if ($response) {
                    (string) $data = $response->getBody();
                    $ent = json_decode($data, true);
                    $keys = array_keys(array_column($ent, "type"), "Relative");
                    foreach ($keys as $key) {
                        unset($ent[$key]);
                    }

                    usort($ent, "cmp_classe");
                    if (count($ent) > 0) {
                        apcu_store('ent', $ent, 600);
                    } else {
                        $html .= "erreur lors de la récupération des données ENT, <br>\n" . print_r($data, true);
                        apcu_store('ent', array(), 600);
                    }
                } else {
                    $html .= "erreur lors de la récupération des données ENT, pas de réponse du serveur<br>\n";
                    apcu_store('ent', array(), 600);
                }
            } catch (\GuzzleHttp\Exception\TransferException $e) {
                $html .= "erreur lors de la connexion à l'ENT<br>\n";
                $html .= GuzzleHTTP\Psr7\str($e->getRequest());
                $html .= GuzzleHTTP\Psr7\str($e->getResponse());
                apcu_store('ent', array(), 600);
            }
        }
    }
    $html .= count($ent) . " utilisateurs synchronisés<br>";
    // $html .= "<pre>" . print_r($ent, true) . "</pre>";

    return $html;
}

function sync_ent_group($config, $uid)
{
    // url liste d'un groupe : /directory/user/group/$uid
}

function sync_ent_groups($config, $type)
{
    // url liste des groupes : https://ent.iledefrance.fr/directory/group/admin/list?type=FunctionalGroup&structureId=0f851f90-9dff-47d0-979e-0feec63db6b6
    // url liste d'un groupe : /directory/user/group/0a6e75a6-74f5-4498-ac18-7054732939f2
}

/**
 * récupère les infos dans un ficihier csv.
 * Attention format UTF-8 obligatoire
 *
 * @param unknown $config
 * @param unknown $csv_file
 * @return array|unknown|unknown[]|string|array|unknown
 */
function import_ent($config, $csv_file, $file_type)
{
    require_once "gpo.inc.php";
    $contents = file_get_contents($csv_file);
    $enc = detect_utf_encoding($contents);
    if ($file_type == "kosmos") {
        $contents = iconv($enc, "UTF-8", $contents);
    } elseif ($file_type == "csv_pronote") {
        $contents = iconv($enc, "UTF-8", $contents);
    }
    // $contents = substr($contents, 3);
    $contents = explode("\n", $contents);
    $data = array();
    foreach ($contents as $num => $line) {
        if ($num == 0) {
            $keys = preg_replace("/\"/", "", explode(";", $line, 60));
            if (count($keys) > 59) {
                array_pop($keys);
            }
            $keys = map_ent_fields($keys, $file_type);
        } elseif (! empty($line)) {
            $index = $num - 1;
            $val = preg_replace("/\"/", "", explode(";", $line, 60));
            if (count($val) > 59) {
                array_pop($val);
            }
            if (count($val) > 0) {
                foreach ($keys as $ind => $key) {
                    if (! empty($key)) {
                        if ($key == "allClasses" || $key == "functions" || $key == "allGroups") {
                            $data[$index][$key] = map_classe(explode(",", $val[$ind]), $file_type);
                        } elseif ($key == "type") {
                            $data[$index][$key] = map_profil($val[$ind], $file_type);
                        } elseif ($key == "birthDate") {
                            $data[$index][$key] = map_birthdate($val[$ind], $file_type);
                        } elseif ($key == "sex") {
                            $data[$index][$key] = map_sex($val[$ind]);
                        } else {
                            if (! empty($val[$ind])) {
                                $data[$index][$key] = $val[$ind];
                            }
                        }
                    }
                }
                if ($file_type == "csv_pronote_eleve") {
                    $data[$index]['type'] = "Student";
                }
            }
        }
    }
    usort($data, "cmp_classe");
    $n = count($data) - 1;
    if ($n == - 1) {
        $n = "aucun";
    }
    $html = "Fichier " . $csv_file . " format " . $file_type . " encodé en $enc : " . $n . " utilisateurs analysés<br>";
    apcu_store('ent', $data, 600);
    // $html .= "<pre>" . print_r($data, true) . "</pre>";
    return $html;
}

function ldap_users($config)
{
    $users = search_ad($config, "*", "user"); // , $config['dn']['people']);
    return $users;
}

/**
 * construit un tableau avec les utilisateurs classés par catégorie en classe
 * pour l'affichage des listes de selection
 *
 * @param unknown $config
 * @param unknown $users
 * @return array|unknown
 */
function ldap_users_group($config, $users)
{
    // return $users;
    $classes = array(
        'eleve' => array(),
        'prof' => array(),
        'administratif' => array()
    );
    foreach ($users as $user) {
        if (strcasecmp(ldap_dn2oudn($user['dn']), $config['dn']['trash']) == 0) {
            $classes['trash'][] = $user;
        } elseif (isset($user['memberof'])) {
            $i = 0;
            if (preg_match("/.*(admin|exam|invite|test).*/i", $user['cn'])) {
                $user['protected'] = true;
            }
            foreach ($user['memberof'] as $groupe) {
                if (preg_match("/CN=no_trash_users.*/", $groupe)) {
                    $user['protected'] = true;
                    break;
                }
            }
            foreach ($user['memberof'] as $groupe) {
                $m = array();
                if (preg_match("/CN=(Classe|Equipe)_(.*),OU=(Classes|Equipes).*/", $groupe, $m)) {
                    if ($m[1] == "Classe") {
                        $i ++;
                        $classes['eleve'][$m[2]][] = $user;
                    } elseif ($m[1] == "Equipe" && $i ++ == 0) {
                        $classes['prof'][] = $user;
                    }
                }
            }
            if ($i == 0) {
                foreach ($user['memberof'] as $groupe) {
                    $m = array();
                    if (preg_match("/CN=(Eleves|Profs|Administratifs),OU=Groups.*/", $groupe, $m)) {
                        if ($m[1] == "Eleves") {
                            $classes['eleve']['sans'][] = $user;
                        } elseif ($m[1] == "Profs") {
                            $classes['prof'][] = $user;
                        } elseif ($m[1] == "Administratifs") {
                            $classes['administratif'][] = $user;
                        }
                    }
                }
            }
        }
    }

    return $classes;
}

function compare_name($ent_string = "", $ad_string = "")
{
    if (empty($ent_string) || empty($ent_string))
        return false;
    $table = [
        "_" => "",
        "-" => "",
        "'" => "",
        " " => ""
    ];
    $ent_string = strtolower(remplace_accents(strtr($ent_string, $table)));
    $ad_string = strtolower(remplace_accents(strtr($ad_string, $table)));
    return ($ent_string == $ad_string);
}

function match_id($config, $ent_user, $ldap_user)
{
    if (! isset($ent_user['externalId']) || ! isset($ldap_user['employeenumber'])) {
        return false;
    } else {
        return ($ent_user['externalId'] == $ldap_user['employeenumber']);
    }
}

function match_birthdate($config, $ent_user, $ldap_user)
{
    if (! isset($ent_user['birthDate']) || ! isset($ldap_user['date'])) {
        return false;
    } else {
        return ($ent_user['birthDate'] == $ldap_user['date']);
    }
}

function match_user($config, $ent_user, $ldap_user)
{
    if (isset($ent_user['login']) && $ent_user['login'] == $ldap_user['cn']) {
        return true;
    } elseif (isset($ent_user['login']) && strtr($ent_user['login'], "-", "_") == $ldap_user['cn']) {
        return true;
    } elseif (isset($ent_user["originalLogin"]) && $ent_user["originalLogin"] == @$ldap_user["userprincipalname"] . "@" . $config['domain']) {
        return true;
    } elseif (isset($ent_user["firstName"]) && isset($ent_user["lastName"]) && mk_cn($config, $ent_user["lastName"], $ent_user["firstName"]) == $ldap_user["cn"]) {
        return true;
    } elseif (isset($ent_user["firstName"]) && isset($ent_user["lastName"]) && isset($ldap_user["nom"]) && compare_name($ent_user["lastName"], $ldap_user["nom"]) && compare_name($ent_user["firstName"], $ldap_user["prenom"])) {
        return true;
    } else {
        return false;
    }
}

function match_ad($config, $auto = false, $merge = false, $manu = false)
{
    $ent = apcu_fetch('ent');
    if ($merge) {
        $ent = apcu_fetch('pronote');
    }
    $ad_users = ldap_users($config);
    $match = $new = array();
    $l = $n = $e = 0;
    $found = false;
    $html = "Recherche des correspondances <br>";
    $i = 0;
    $rename = 0;
    foreach ($ent as $e => $ent_user) {
        if ($found) {
            unset($ad_users[$l]);
        }
        $found = false;
        // $html .= "Compte ENT " . $ent_user['firstName'] . " " . $ent_user['lastName'] . "(" . $ent_user['login'] . ") : ";
        if ($ent_user['type'] != "Relative") {
            $ent_user['birthDate'] = map_birthdate($ent_user['birthDate']);
            foreach ($ad_users as $l => $ad_user) {
                if (match_id($config, $ent_user, $ad_user)) {
                    $match[$i] = [
                        'ent' => $ent_user,
                        'ad' => $ad_user
                    ];
                    // $i ++;
                    $found = true;
                    break;
                }
            }
            if (! $found) {
                foreach ($ad_users as $l => $ad_user) {

                    if (match_user($config, $ent_user, $ad_user) && match_birthdate($config, $ent_user, $ad_user)) {
                        $match[$i] = [
                            'ent' => $ent_user,
                            'ad' => $ad_user
                        ];
                        // $i ++;
                        $found = true;
                        break;
                    }
                }
            }
            if (! $found) {
                foreach ($ad_users as $l => $ad_user) {

                    if (match_user($config, $ent_user, $ad_user)) {
                        $match[$i] = [
                            'ent' => $ent_user,
                            'ad' => $ad_user
                        ];
                        // $i ++;
                        $found = true;
                        break;
                    }
                }
            }
            if ($found && ! $manu) {
                if ($config['ent_autorename'] == "1") {
                    // if ((($ent_user['originalLogin'] == $ent_user['login'] || $config['force_ad_alias']) && ($ent_user['login'] != $ad_user['cn'])) || strlen($ad_user['cn']) > 19) { // || mk_cn($config, $ent_user['lastName'], $ent_user['firstName'], true) != $ad_user['cn']) {
                    $alias = false;
                    if (strlen($ad_user['cn']) > 19) {
                        $alias = mk_cn($config, $ent_user['lastName'], $ent_user['firstName']);
                        $msg = "  <b>" . $alias . "</b> car le le login est trop long.";
                    } elseif (! empty($ent_user['login']) && ! empty($ent_user['originalLogin']) && $ent_user['login'] != $ent_user['originalLogin'] && $ent_user['login'] != $ad_user['cn']) {
                        // un alias est défini, on le garde
                        $alias = $ent_user['login'];
                        $msg = ", on tente de garder : <b>" . $alias . "</b>.";
                    } elseif (! empty($ent_user['login']) && ! empty($ent_user['originalLogin']) && $ent_user['login'] == $ent_user['originalLogin'] && $ent_user['originalLogin'] != $ad_user['cn']) {
                        $alias = mk_cn($config, $ent_user['lastName'], $ent_user['firstName']);
                        if ($alias == $ad_user['cn']) {
                            $msg = ", on tente de garder l'AD :  <b>" . $alias . "</b>.";
                        } else {
                            $msg = ", on tente de simplifier en : <b>" . $alias . "</b>.";
                        }
                    } else {
                        $alias = false;
                    }
                    // if ((! empty($ent_user['login']) && $ent_user['login'] != $ad_user['cn'] && ! empty($ent_user['originalLogin']) && $ent_user['originalLogin'] != $ad_user['cn']) || strlen($ad_user['cn']) > 19) { // || mk_cn($config, $ent_user['lastName'], $ent_user['firstName'], true) != $ad_user['cn']) {
                    if ($alias) {

                        $match[$i]['rename'] = $alias;
                        $html .= "Compte ENT <b> " . $ent_user['firstName'] . " " . $ent_user['lastName'] . "</b> (ENT " . $ent_user['originalLogin'] . "/" . $ent_user['login'] . ", AD " . $ad_user['cn'] . ") : ";
                        $html .= " à renommer" . $msg . "<br>";
                        $rename ++;
                    }
                }
                $i ++;
            } else {

                $new_login = mk_cn($config, $ent_user['lastName'], $ent_user['firstName']);
                if (! isset($ent_user['login'])) {
                    $ent_user['login'] = $new_login;
                }
                if ((! empty($ent_user['allClasses'][0]['name']) && ! empty($ent_user['type']) && $auto) || $manu) {
                    $match[$i ++] = [
                        'ent' => $ent_user,
                        'ad' => array()
                    ];
                    $n ++;
                    $html .= "Compte ENT " . $ent_user['firstName'] . " " . $ent_user['lastName'] . "(" . $ent_user['login'] . ") : ";
                    $html .= " nouvel utilisateur, login demandé : " . $new_login . "<br>";
                } else {
                    $new[] = $ent_user;
                    // $html .= " pas de classe : non traité.<br>";
                }
            }
        } else {
            // $html .= " compte parent, non traité.<br>";
        }
    }
    $classes = ldap_users_group($config, $ad_users);
    $e = 0;
    foreach ($classes['eleve'] as $classe) {
        $e += count($classe);
    }
    $html .= "synchronisation : " . (count($match) - $n) . " utilisateurs ont été mis en correspondance automatiquement<br>";
    $html .= "Nouveaux utilisateurs : " . $n . " comptes.<br>";
    $html .= "Comptes à renommer : " . $rename . "<br>";
    // $html .= "Comptes à traiter manuellement : " . count($unmatch) . "<br>";
    $html .= "Comptes AD sambaEdu sans correspondance ENT : profs : " . count($classes['prof']) . "   Eleves : " . $e . "  Administratifs : " . count($classes['administratif']) . "<br>";
    $html .= "Comptes ENT sans correspondance dans AD SambaEdu, à traiter manuellement : " . count($new) . "<br>";
    apcu_store('ad', $ad_users, 600);
    apcu_store('new', $new, 600);
    apcu_store('match', $match, 600);
    apcu_store('ent_count', count($match), 1000);
    // $html .= "<pre>" . print_r($match, true) . "</pre>";
    return $html;
}

function merge_pronote($config)
{
    $m = 0;
    $html = match_ad($config, true);
    $ent_match = apcu_fetch('ent');
    $ent_new = apcu_fetch('new');
    $html .= match_ad($config, true, true);
    $pronote_match = apcu_fetch('ent');
    $pronote_new = apcu_fetch('new');
    foreach ($ent_match as $i => $ent_user) {
        if ($ent_user['ent']['type'] == "Teacher") {
            foreach ($pronote_match as $pronote_user) {
                if ($pronote_user['ent']['type'] == "Teacher" && $pronote_user['ent']['login'] == $ent_user['ent']['login']) {
                    $ent_match[$i]['ent']['allClasses'] = $pronote_user['ent']['allClasses'];
                    $m ++;
                    break;
                }
            }
        }
    }
    foreach ($ent_new as $j => $ent_user) {
        if ($ent_user['type'] == "Teacher") {
            foreach ($pronote_new as $pronote_user) {
                if ($pronote_user['type'] == "Teacher" && $pronote_user['login'] == $ent_user['login']) {
                    $ent_user['allClasses'] = $pronote_user['allClasses'];
                    $ent_match[] = [
                        'ent' => $ent_user,
                        'ad' => array()
                    ];
                    $m ++;
                    unset($ent_new[$j]);
                    break;
                }
            }
        }
    }

    $html .= "fusion pronote : " . $m . " comptes profs ont été fusionnés<br>";
    $html .= "Comptes ENT sans correspondance dans AD SambaEdu, à traiter manuellement : " . count($new) . "<br>";
    apcu_store('new', $ent_new, 600);
    apcu_store('match', $ent_match, 600);
    return $html;
}

function match_ent_users($ad_cns)
{
    $match = apcu_fetch('match');
    $ent = apcu_fetch('new');
    $ad = apcu_fetch('ad');
    $n = $m = 0;
    $html = count($ad_cns) . "correspondances manuelles : <br>";
    foreach ($ad_cns as $login => $ad_cn) {
        // $html .= "$login => $ad_cn<br>";
        if (! empty($ad_cn) && $ad_cn != "admin") {
            if ($ad_cn == "#new#") {
                // nouveau
                $ad_user = array();
                $n ++;
            } else {
                foreach ($ad as $k => $ad_user) {
                    if ($ad_user['cn'] == $ad_cn) {
                        break;
                    }
                }
                $html .= "on retire $k => " . $ad_user['cn'] . "<br>";
                unset($ad[$k]);
                $m ++;
            }
            foreach ($ent as $index => $ent_user) {
                if ($ent_user['login'] == $login) {
                    break;
                }
            }
            $match[] = [
                'ent' => $ent_user,
                'ad' => $ad_user
            ];
            unset($ent[$index]);
        }
    }
    apcu_store('ad', $ad, 600);
    apcu_store('ent', $ent, 600);
    apcu_store('match', $match, 600);
    $count = count($match) ?? 0;
    $html .= "Mise en correspondance des comptes :<br>\n";
    $html .= "$m comptes mis en correspondance et $n nouveaux comptes à créer.<br>\n";
    $html .= "Il reste " . $count . " correspondances à traiter<br>";
    return $html;
}

function rename_ent_users($ad_cns)
{
    $match = apcu_fetch('match');
    $m = 0;
    $html = count($ad_cns) . "renommage de :";
    foreach ($ad_cns as $login => $ad_cn) {
        foreach ($match as $index => $user) {
            if ($user['ent']['login'] == $login) {
                $match[$index]['rename'] = $login;
                $html .= " " . $login . ",";
                $m ++;
                break;
            }
        }
    }
    apcu_store('match', $match, 600);

    $html .= "<br>$m compte(s) à renommer.<br>\n";
    return $html;
}

function trash_ad_users($ad_cns)
{
    $users = apcu_fetch('ad');
    $match = apcu_fetch('match');
    $m = 0;
    $html = "mise à la corbeille de : ";
    foreach ($ad_cns as $login => $ad_cn) {
        foreach ($users as $l => $user) {
            if ($user['cn'] == $login) {
                $users[$l]['trash'] = true;
                $trash = [
                    'ad' => $users[$l],
                    'ent' => []
                ];
                if (! in_array($trash, $match)) {
                    array_unshift($match, $trash);
                    $html .= " " . $login . ",";
                    $m ++;
                }
                break;
            }
        }
    }
    apcu_store('ad', $users, 600);
    $match = apcu_store('match', $match, 600);
    $html .= "<br>$m compte(s) vont être mis à la corbeille.<br>\n";
    return $html;
}

function create_ent_user($config, &$user, &$html)
{
    $random = create_random_password(8, true);
    $command = "user create '" . $user['ent']['login'] . "' ";
    if (@$config['no_passwd_change'] == "1") {
        $passwd_must_change = " ";
    } else {
        $passwd_must_change = " --must-change-at-next-login ";
    }
    if (! empty($user['ent']['code'])) {
        // on fixe le password initial à la valeur du code ent, et on doit le changer...
        $command .= escapeshellarg($user['ent']['code']) . $passwd_must_change;
        $password = $user['ent']['code'];
    } else {
        switch ($config['pwdPolicy']) {
            case "0":
            case "1":
                if (! empty($user['ent']['birthDate'])) {
                    $password = implode(array_reverse(explode("-", $user['ent']['birthDate'])));
                } else {
                    $password = $config['activation_code'] ?? $random;
                }
                $command .= escapeshellarg($password) . $passwd_must_change;
                break;
            case "2":
            case "3":
                $password = $config['activation_code'] ?? $random;
                $command .= escapeshellarg($password) . " --must-change-at-next-login ";
                break;
            default:
                $command .= $random . " --must-change-at-next-login ";
        }
    }
    $sex = $user['ent']['sex'] ?? "M";
    if (! empty($user['ent']['birthDate'])) {
        $command .= "--physical-delivery-office='" . str_replace("-", "", $user['ent']['birthDate']) . "," . $sex . "' ";
    }
    $categorie = "";
    switch ($user['ent']['type']) {
        case "Student":
            $categorie = "Eleves";
            break;
        case "Teacher":
            $categorie = "Profs";
            break;
        case "Personnel":
        case "Guest":
            $categorie = "Administratifs";
            break;
    }
    $prenom = escapeshellarg($user['ent']['firstName']);
    $nom = escapeshellarg($user['ent']['lastName']);
    $config['ent_email'] = $config['ent_email'] ?? 0;
    if ($config["ent_email"] == "1" && ! empty($user['ent']['email'])) {
        $email = $user['ent']['email'] ?? $user['ent']['login'] . "@" . $config['domain'];
    } else {
        $email = $user['ent']['login'] . "@" . $config['domain'];
    }
    if (empty($user['ent']['externalId'])) {
        $command .= "--use-username-as-cn --given-name=$prenom --surname=$nom --mail-address='" . $email . "' ";
    } else {
        $command .= "--job-title='" . $user['ent']['externalId'] . "' --use-username-as-cn --given-name=$prenom --surname=$nom --mail-address='" . $email . "' ";
    }
    if (! empty($categorie)) {
        $command .= " --userou='ou=" . $categorie . "," . $config['people_rdn'] . "'";
        $message = array();
        $result = sambatool($config, $command, $message);
        // $html .= "<pre>" . $command . "</pre><br>";
        if ($result == 0) {
            $user['ad'] = search_user($config, $user['ent']['login']);
            // Ajout a un groupe principal
            $result = groupaddmember($config, $user['ad']['cn'], $categorie);
            if ($result)
                $html .= "Utilisateur ajouté au groupe " . $categorie . "<br>";
            else
                $html .= "ERREUR Utilisateur pas ajouté à " . $categorie . "<br>";
            $html .= "Utilisateur type : " . $categorie . " : " . $user['ad']['cn'] . " mot de passe initial " . $password . " créé,<br>";
            $user['ent']['code'] = $password;
        } else {
            $html .= "ERREUR l'utilisateur  type : " . $categorie . " : " . $user['ent']['login'] . " n'a pas été créé<br>" . $command . "<br>" . implode("<br>", $message) . "<br>\n";
            $result = false;
        }
    } else {
        $html .= "ERREUR l'utilisateur  " . $user['ent']['login'] . " n'a pas été créé car il n'est pas conforme<br>";
        $result = false;
    }
    return $result;
}

function modify_ad_user($config, $user, &$html)
{
    $modif = [];
    $config['ent_email'] = $config['ent_email'] ?? 0;

    foreach ($user['ent'] as $ent_attr => $ent_val) {
        switch ($ent_attr) {
            case 'externalId':
                if (! isset($user['ad']['employeenumber'])) {
                    $modif[] = [
                        "attrib" => "title",
                        "modtype" => LDAP_MODIFY_BATCH_ADD,
                        "values" => [
                            $ent_val
                        ]
                    ];
                } elseif ($user['ad']['employeenumber'] != $ent_val) {
                    $modif[] = [
                        "attrib" => "title",
                        "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                        "values" => [
                            $ent_val
                        ]
                    ];
                }
                break;
            case 'originalLogin':
                if (! isset($user['ad']['userprincipalname'])) {
                    $modif[] = [
                        "attrib" => "userprincipalname",
                        "modtype" => LDAP_MODIFY_BATCH_ADD,
                        "values" => [
                            $ent_val . "@" . $config['domain']
                        ]
                    ];
                } elseif ($user['ad']['userprincipalname'] != $ent_val . "@" . $config['domain']) {
                    $modif[] = [
                        "attrib" => "userprincipalname",
                        "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                        "values" => [
                            $ent_val . "@" . $config['domain']
                        ]
                    ];
                }
                break;
            case 'firstName':
                if (! isset($user['ad']['prenom'])) {
                    $modif[] = [
                        "attrib" => "givenname",
                        "modtype" => LDAP_MODIFY_BATCH_ADD,
                        "values" => [
                            $ent_val
                        ]
                    ];
                } elseif ($user['ad']['prenom'] != $ent_val) {
                    $modif[] = [
                        "attrib" => "givenname",
                        "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                        "values" => [
                            $ent_val
                        ]
                    ];
                }
                break;
            case 'email':
                if (@$config['ent_email'] == 1) {
                    if (empty($ent_val)) {
                        $ent_val = $user['ent']['login'] . "@" . $config['domain'];
                    }
                    if (! isset($user['ad']['email'])) {
                        $modif[] = [
                            "attrib" => "mail",
                            "modtype" => LDAP_MODIFY_BATCH_ADD,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    } elseif ($user['ad']['email'] != $ent_val) {
                        $modif[] = [
                            "attrib" => "mail",
                            "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    }
                }
                break;
            case 'lastName':
                if (! isset($user['ad']['nom'])) {
                    $modif[] = [
                        "attrib" => "sn",
                        "modtype" => LDAP_MODIFY_BATCH_ADD,
                        "values" => [
                            $ent_val
                        ]
                    ];
                } elseif ($user['ad']['nom'] != $ent_val) {
                    $modif[] = [
                        "attrib" => "sn",
                        "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                        "values" => [
                            $ent_val
                        ]
                    ];
                }
                break;
            case 'displayName':
                if (! isset($user['ad']['fullname'])) {
                    $modif[] = [
                        "attrib" => "displayname",
                        "modtype" => LDAP_MODIFY_BATCH_ADD,
                        "values" => [
                            $ent_val
                        ]
                    ];
                } elseif ($user['ad']['fullname'] != $ent_val) {
                    $modif[] = [
                        "attrib" => "displayname",
                        "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                        "values" => [
                            $ent_val
                        ]
                    ];
                }
                break;
            // pour le moment on s'arrête là, on pourrait completer d'autres attributs...
        }
    }
    if (count($modif) == 0) {
        $result = true;
    } else {
        $dn = $user['ad']['dn'];
        $ds = bind_ad_gssapi($config);
        if ($ds) {
            $result = ldap_modify_batch($ds, $dn, $modif);
        }
        @ldap_close($ds);
        if ($result)
            $html .= "Utilisateur " . $user['ad']['cn'] . " mis à jour <br>";
        else {
            $html .= "ERREUR : Utilisateur " . $user['ad']['cn'] . " pas mis à jour <br>";
        }
    }

    return $result;
}

function modify_ad_login($config, $user, &$html)
{
    $old = $user['ad']['cn'];
    if (count(search_user($config, $user['ent']['login'])) > 0) {
        $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " est déjà en doublon dans l'AD, veuillez l'effacer manuellement ou choisir un login ENT différent<br>";
        return false;
    }
    $result = move_ad($config, $user['ad']['cn'], "cn=" . $user['ent']['login'] . "," . ldap_dn2oudn($user['ad']['dn']), "user");
    if ($result) {
        $attrs = array();
        $attrs['samaccountname'] = $user['ent']['login'];
        $result = modify_ad($config, $user['ent']['login'], "user", $attrs);
        system("sudo mv /home/" . $user['ad']['cn'] . " /home/" . $user['ent']['login']);
        system("sudo mv /home/profiles/" . $user['ad']['cn'] . ".V6 /home/profiles/" . $user['ent']['login'] . ".V6");
        if (is_eleve($config, $user['ent']['login'])) {
            $old_rep = invert_login($config, $user['ad']['cn']);
            $new_rep = invert_login($config, $user['ent']['login']);
            foreach (list_classes($config, $user['ent']['login']) as $classe) {
                system("sudo mv '/var/sambaedu/Classes/Classe_" . $classe . "/" . $old_rep . "' '/var/sambaedu/Classes/Classe_" . $classe . "/" . $new_rep . "'");
            }
        }
        if ($old != $user['ad']['cn']) {
            $html .= "Utilisateur " . $old . " renommé " . $user['ad']['cn'];
        }
    } else {
        $html .= "ERREUR : Utilisateur " . $user['ad']['cn'] . " non renommé <br>";
    }
    return $result;
}

/**
 * crée ou met à jour un utilisateur à partir de du tableau des correspondances ent.
 * utilise le tableau 'match' en cache apcu, et enleve l'utilisateur en cas de succès.
 *
 * @param unknown $config
 * @return string
 */
function update_user_ent($config, &$html, $remove = false, $manu = false)
{
    $client = NULL;
    $result = true;
    $match = apcu_fetch('match');
    $group = apcu_fetch('group');
    $ad = apcu_fetch('ad');
    if (count($match) > 0) {
        $user = array_shift($match);
        // $html .= "Utilisateur " . $user['ent']['login'] . "<br>";
        $rename = $user['rename'] ?? false;
        $full_login = $user['ent']['login'] ?? "";
        if (empty($full_login) && $remove) {
            if (isset($user['ad']['cn'])) {
                if (trash_user($config, $user['ad'])) {
                    $html .= "Deplacement dans la corbeille de l'utilisateur " . $user['ad']['cn'] . "<br>";
                    $result = true;
                } else {
                    $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " n'a pas été mis à la corbeille<br>";
                    $result = false;
                }
            } else {
                $html .= "ATTENTION : Utilisateur inconnu <br>";
                $result = false;
            }
        } elseif (! isset($user['ad']['cn'])) {
            // nouvel utilisateur, on met en place les attributs de base, les autres seront synchronisés automatiquement
            $user['ad'] = search_user($config, $user['ent']['login']);
            if (count($user['ad']) > 0) {
                $result = false;
                $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " est en doublon dans L'ENT<br>";
            } else {
                $html .= "Nouvel Utilisateur " . $user['ent']['login'] . "<br>";
                change_ent_alias($config, $client, $ad, $user, $html, $manu);
                $result = create_ent_user($config, $user, $html);
                $user['ad'] = search_user($config, $user['ent']['login']);
                if (count($user['ad']) == 0) {
                    $result = false;
                    $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " non créé <br>";
                } else {
                    if ($full_login == $user['ent']['login']) {
                        $html .= "Utilisateur " . $user['ad']['cn'] . " créé<br>";
                    } else {
                        $html .= "Alias de login modifié pour l'utilisateur " . $full_login . "=>" . $user['ad']['cn'] . "<br>";
                    }
                }
            }
        } else {
            if (strcasecmp(ldap_dn2oudn($user['ad']['dn']), $config['dn']['trash']) == 0) {
                $categorie = "";
                switch ($user['ent']['type']) {
                    case "Student":
                        $categorie = "Eleves";
                        break;
                    case "Teacher":
                        $categorie = "Profs";
                        break;
                    case "Personnel":
                    case "Guest":
                        $categorie = "Administratifs";
                        break;
                }
                $result = recup_user($config, $user['ad']['cn'], $categorie);
                if ($result) {
                    $user['ad'] = search_user($config, $user['ad']['cn']);
                    $html .= "Utilisateur " . $user['ad']['cn'] . " récupéré <br>";
                } else {
                    $html .= "ERREUR : Utilisateur " . $user['ad']['cn'] . " non récupéré <br>";
                }
            }
            if ($rename) { // && $user['ent']['login'] != $user['ad']['cn']) {
                $html .= "renommage de " . $user['ad']['cn'] . "<br>";
                if (! change_ent_alias($config, $client, $ad, $user, $html)) {
                    // modif ent impossible, on change AD
                    $result = modify_ad_login($config, $user, $html);
                    if ($result)
                        $html .= "changement d'alias ENT impossible, on change juste l'AD " . $full_login . "=>" . $user['ad']['cn'] . "<br>";
                } elseif ($user['ad']['cn'] != $user['ent']['login']) {
                    // modif ent ok, mais resultat different, on change AD
                    $result = modify_ad_login($config, $user, $html);
                    if ($result)
                        $html .= " Alias ENT different, on change l'AD " . $user['ad']['cn'] . " => " . $full_login . "<br>";
                }
                $user['ad'] = search_user($config, $user['ent']['login']);
            }
            $result = modify_ad_user($config, $user, $html);
        }
        // gestion des classes
        $group[] = $user;

        if ($result) {
            apcu_store('group', $group, 600);
        }
        apcu_store('match', $match, 600);

        return count($match);
    } else {
        return false;
    }
}

/**
 * crée ou met à jour ls groupes un utilisateur à partir de du tableau des correspondances ent.
 * utilise le tableau 'group' en cache apcu, et enleve l'utilisateur en cas de succès.
 *
 * @param unknown $config
 * @return string
 */
function update_group_ent($config, &$html, $remove = false)
{
    $time = time();
    $client = NULL;
    $result = false;
    $end = true;
    $match = apcu_fetch('group');
    $ad = apcu_fetch('ad');
    if (count($match) > 0) {
        $user = array_shift($match);
        if (! isset($user['diff'])) {
            $ent_classes = array();
            if (@count($user['ent']['allClasses']) > 0) {
                foreach ($user['ent']['allClasses'] as $ent_groups) {
                    $ent_classes[] = $ent_groups['name'];
                }
            }
            $ad_classes = array();
            if (@count($user['ad']['memberof']) > 0) {
                foreach ($user['ad']['memberof'] as $ad_group) {
                    $m = array();
                    if (preg_match("/(Classe|Equipe)_(.*)/", ldap_dn2cn($ad_group), $m)) {
                        $ad_classes[] = $m[2];
                    }
                }
            }
            $user['diff']['add'] = array_diff($ent_classes, $ad_classes);
            $user['diff']['remove'] = array_diff($ad_classes, $ent_classes);
            // $html .= print_r($diff);
        }
        if ($remove && (! empty($config['ent_autogroups']) || $user['ent']['type'] != 'Teacher')) {
            if (count($user['diff']['remove']) > 0) {
                foreach ($user['diff']['remove'] as $key => $classe) {
                    if (! empty($classe)) {
                        $result = remove_user_group($config, $classe, $user['ad']['cn'], false);
                        if ($result) {
                            $html .= $user['ad']['cn'] . " enlevé de la classe " . $classe . ", <br>";
                            unset($user['diff']['remove'][$key]);
                        } else {
                            $html .= "ERREUR " . $user['ad']['cn'] . " n'a pas été enlevé de la classe " . $classe . ", <br>";
                        }
                    }
                    if ((time() - $time) > 15) {
                        $end = false;
                        break;
                    }
                }
            }
        }
        if ($end && count($user['diff']['add']) > 0) {
            foreach ($user['diff']['add'] as $key => $classe) {
                if (! empty($classe)) {
                    if (! type_group($config, $classe)) {
                        create_group($config, $classe, "Classe " . $classe, "classe");
                    }
                    $result = add_user_group($config, $classe, $user['ad']['cn'], true);
                    if ($result) {
                        $html .= $user['ad']['cn'] . " ajouté à la classe " . $classe . ", <br>";
                        unset($user['diff']['add'][$key]);
                    } else {
                        $html .= "ERREUR " . $user['ad']['cn'] . " n'a pas été ajouté à la classe " . $classe . ", <br>";
                    }
                }
                if ((time() - $time) > 20) {
                    $end = false;
                    break;
                }
            }
        }
        if (! $end) {
            $html .= "Attention, " . $user['ad']['cn'] . " traitement trop long interrompu<br>";
            array_unshift($match, $user);
        }
        apcu_store('group', $match, 600);
        return count($match);
    } else {
        return false;
    }
}

/**
 * applique les changements dans l'ad.
 * Peut être long...
 *
 * @todo rendre la fonction asynchrone, avec rappel de la page jusqu'à épuisement du le la liste des modifications
 *       executer le traitement par lots de 100 utilisateurs.
 *       on limite l'execution à 20s pour eviter les timeouts dans l'interface
 * @param unknown $config
 * @return string
 */
function update_ad_from_ent($config, $duree, $remove = false)
{
    $match = apcu_fetch('match');
    $html = apcu_fetch('ent_html') ?? "<b>Début de la mise à jour de la base </b><br>";
    $nb = 0;
    $res = $size = count($match);
    $time = time();
    while ($res > 0 && $nb ++ < $size && (time() - $time < $duree)) {
        $res = update_user_ent($config, $html, $remove);
    }
    $type = "utilisateurs";
    $match = apcu_fetch('group');
    if ($res == 0 && $match) {
        $nb = 0;
        $res = $size = count($match);
        while ($res > 0 && $nb ++ < $size && (time() - $time < $duree)) {
            $res = update_group_ent($config, $html, $remove);
        }
        $type = "groupes";
    }
    // $html .= $res . " ";
    if ($res > 0) {
        $end = false;
    } else {
        $end = true;
        $html .= "<b>Import terminé !</b>\n";
    }
    apcu_store('ent_count', $res, 600);
    apcu_store('ent_html', $html);
    apcu_store('ent_type', $type, 600);

    return $end;
}

// //////////////////////////////////////////////////////////////////////////////////////////
function form_config_ent($config, $ent, $file = false)
{
    // liste du parametrage par defaut des ent à compléter
    $ent_list = array(
        'openent' => array(
            'description' => "ENT openent",
            'mode' => "openent",
            'auth' => 1,
            'uri' => array(
                'idf' => "https://ent.iledefrance.fr/",
                'paris' => "https://www.parisclassenumerique.fr/",
                'essonne' => "https://www.moncollege-ent.essonne.fr/"
            ),
            'user' => "",
            'user_password' => "",
            'oauth2_id' => "sambaedu",
            'oauth2_pass' => "",
            'oauth2_redirect_uri' => "https://se4." . $config['domain'] . "/oauth2/callback.php",
            'oauth2_auth' => "auth/oauth2/auth",
            'oauth2_token' => "auth/oauth2/token",
            'oauth2_userinfo' => "auth/oauth2/userinfo",
            'cron' => 0
        ),
        'kosmos' => array(
            'description' => "Fichier export ENT kosmos",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv_openent' => array(
            'description' => "Fichier export csv OpenENT",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv_pronote' => array(
            'description' => "Fichier export csv EDT/Pronote (eleves ou professeurs)",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv' => array(
            'description' => "Aucun (csv)",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        )
        /*
     * 'pronote' => array(
     * 'description' => "Pronote.net (abonnement)",
     * 'mode' => "pronote",
     * 'auth' => 0,
     * 'api_url' => "https://pronote-api.sambaedu.org",
     * 'api_user' => "synchro",
     * 'api_passwd' => "",
     * 'server_url' => "https://pronote.diderot.org/",
     * 'cas' => [
     * "ac-besancon",
     * "ac-caen",
     * "ac-clermont",
     * "ac-grenoble",
     * "ac-lille",
     * "ac-lyon",
     * "ac-montpellier",
     * "ac-nancy-metz",
     * "ac-nantes",
     * "ac-orleans-tours",
     * "ac-reims",
     * "ac-rouen",
     * "ac-strasbourg",
     * "ac-toulouse",
     * "arsene76",
     * "aten",
     * "hdf",
     * "iledefrance",
     * "kdecole-wayf",
     * "none",
     * "seineetmarne",
     * "toutatice",
     * "viescolaire"
     * ],
     * 'cours_unit' => "55",
     * 'cron' => 0
     * )
     */
    );
    // menu de choix de l'ent à configurer
    if (empty($ent)) {
        $ent['ent_type'] = "csv";
    }
    $html = "Choix de l'ENT ";
    $html .= "<SELECT NAME=\"ent[ent_type]\" onchange=submit()>";
    $html .= "<option value=\"\">Choisir...</option>";
    foreach ($ent_list as $type => $params) {
        if ($file && $params['mode'] == "csv" || ! $file) {
            $html .= "<option value=\"$type\"";
            if ($type == $ent['ent_type']) {
                $html .= " selected>";
            } else {
                $html .= ">";
            }
            $html .= $params['description'] . "</option>";
        }
    }
    $html .= "</SELECT><br>";
    if (! $file) {
        if (! isset($ent['pronote_cron']))
            $ent['pronote_cron'] = "0";
        if (! isset($ent['openent_cron']))
            $ent['openent_cron'] = "0";
        if (! isset($ent['ent_autorename']))
            $ent['ent_autorename'] = "0";
        if (! isset($ent['ent_autogroups']))
            $ent['ent_autogroups'] = "0";
        if (! isset($ent['ent_auth']))
            $ent['ent_auth'] = "0";

        foreach ($ent as $key => $val) {
            if (isset($val) && $val != $config[$key]) {
                set_config($key, $val, "ent");
            }
        }
        $config = get_config(true);
    } else {
        $config['ent_type'] = $ent['ent_type'];
    }
    if (isset($config['ent_type'])) {

        foreach ($ent_list[$config['ent_type']] as $key => $val) {
            $prefix = $ent_list[$config['ent_type']]['mode'] . "_";
            $ent[$prefix . $key] = $config[$prefix . $key] ?? $val;
        }
        switch ($ent_list[$ent['ent_type']]['mode']) {
            case "pronote":
                // if (! isset($ent['pronote_autorename']))
                // $ent['pronote_autorename'] = "0";

                $html .= "<div>URL du serveur passerelle vers Pronote";
                $html .= " <input type=\"text\" name=\"ent[pronote_api_url]\" value=" . $ent['pronote_api_url'] . "><br>\n";
                $html .= "Login du compte vie scolaire de l'établissement utilisé pour la synchro";
                $html .= " <input type=\"text\" name=\"ent[pronote_api_user]\" value=" . $ent['pronote_api_user'] . "><br>\n";
                $html .= "Mot de passe de l'utilisateur de l'etablissement ";
                $html .= " <input type=\"password\" name=\"ent[pronote_api_passwd]\" value=" . $ent['pronote_api_passwd'] . "><br>\n";
                $html .= "URL du serveur Pronote";
                $html .= " <input type=\"text\" name=\"ent[pronote_server_url]\" value=" . $ent['pronote_server_url'] . "><br>\n";
                $html .= "Type de CAS pour l'authentification Pronote";
                $html .= "<SELECT NAME=\"ent[pronote_cas]\">";
                $html .= "<option value=\"\">Choisir...</option>";
                foreach ($ent_list['pronote']['cas'] as $key => $cas) {
                    $html .= "<option value=\"$cas\"";
                    if ($cas == $ent['pronote_cas']) {
                        $html .= " selected>";
                    } else {
                        $html .= ">";
                    }
                    $html .= $cas . "</option>";
                }
                $html .= "</SELECT><br>";
                $html .= "Durée de la séquence de cours";
                $html .= " <input type=\"text\" name=\"ent[pronote_cours_unit]\" value=" . $ent['pronote_cours_unit'] . "><br>\n";

                $html .= "Mise à jour automatique ";
                if ($config['pronote_cron'] == 1)
                    $html .= " <input type=\"checkbox\" name=\"ent[pronote_cron]\" value=\"1\" checked><br>\n";
                else
                    $html .= " <input type=\"checkbox\" name=\"ent[pronote_cron]\"  value=\"1\"><br>\n";
                // On ne connait pas le login pronote ?
                // $html .= "Renommage automatique des logins pour unicité avec l'ent ";
                // if ($config['pronote_autorename'] == 1)
                // $html .= " <input type=\"checkbox\" name=\"ent[pronote_autorename]\" value=\"1\" checked><br>\n";
                // else
                // $html .= " <input type=\"checkbox\" name=\"ent[pronote_autorename]\" value=\"1\"><br>\n";
                break;

            case "csv":
            case "xml":
            case "json":
                $html .= "Format du fichier d'importation des comptes : " . $ent_list[$ent['ent_type']]['mode'] . "<br>";
                break;
            case "openent":

                $html .= "<div>URI de l'ENT ";
                $html .= "<SELECT NAME=\"ent[openent_uri]\">";
                $html .= "<option value=\"\">Choisir...</option>";
                foreach ($ent_list['openent']['uri'] as $key => $uri) {
                    $html .= "<option value=\"$uri\"";
                    if ($uri == $ent['openent_uri']) {
                        $html .= " selected>";
                    } else {
                        $html .= ">";
                    }
                    $html .= $key . " : " . $uri . "</option>";
                }
                $html .= "</SELECT><br>";
                $html .= "Login de l'administrateur ENT de l'etablissement ";
                $html .= " <input type=\"text\" name=\"ent[openent_user]\" value=" . $ent['openent_user'] . "><br>\n";
                $html .= "Mot de passe de l'administrateur ENT de l'etablissement ";
                $html .= " <input type=\"password\" name=\"ent[openent_user_password]\" value=" . $ent['openent_user_password'] . "><br>\n";
                $html .= "<br>Mise à jour automatique : chaque nuit les comptes seront synchronisés avec L'ENT";
                if ($config['openent_cron'] == 1)
                    $html .= " <input type=\"checkbox\" name=\"ent[openent_cron]\" value=\"1\" checked><br>\n";
                else
                    $html .= " <input type=\"checkbox\" name=\"ent[openent_cron]\"  value=\"1\"><br>\n";
                $html .= "<br>Renommage automatique des logins pour unicité avec l'ent : si besoin un alias de login sera créé sur l'ENT pour respecter les contraintes du login sambaedu. De même si l'utilisateur modifie manuellement son login sur l'ENT, la modification sera faite sur sambaedu ";
                if ($config['ent_autorename'] == 1)
                    $html .= " <input type=\"checkbox\" name=\"ent[ent_autorename]\" value=\"1\" checked><br>\n";
                else
                    $html .= " <input type=\"checkbox\" name=\"ent[ent_autorename]\"  value=\"1\"><br>\n";
                $html .= "<br>Synchronisation automatique des equipes et groupes. A n'activer qu'une fois que les equipes et les groupes ont bien été remontés sur L'ENT.
En début d'année, si l'import des équipe et groupe a été faite à l'aide des fichier CSV pronote, ne pas cocher la case ! ";
                if ($config['ent_autogroups'] == 1)
                    $html .= " <input type=\"checkbox\" name=\"ent[ent_autogroups]\" value=\"1\" checked><br>\n";
                else
                    $html .= " <input type=\"checkbox\" name=\"ent[ent_autogroups]\"  value=\"1\"><br>\n";

                $html .= "</div><br><div>Id Oauth2 ";
                $html .= " <input type=\"text\" name=\"ent[openent_oauth2_id]\" value=" . $ent['openent_oauth2_id'] . "><br>\n";
                $html .= "Secret Oauth2 ";
                $html .= " <input type=\"password\" name=\"ent[openent_oauth2_pass]\" value=" . $ent['openent_oauth2_pass'] . "><br>\n";
                $html .= "URI redirect Oauth2 sambaedu ";
                $html .= " <input type=\"text\" size=\"50\" name=\"ent[openent_oauth2_redirect_uri]\" value=" . $ent['openent_oauth2_redirect_uri'] . "><br>\n";
                $html .= "chemin api auth Oauth2 ";
                $html .= " <input type=\"text\" name=\"ent[openent_oauth2_auth]\" value=" . $ent['openent_oauth2_auth'] . "><br>\n";
                $html .= "chemin api token Oauth2 ";
                $html .= " <input type=\"text\" name=\"ent[openent_oauth2_token]\" value=" . $ent['openent_oauth2_token'] . "><br>\n";
                $html .= "chemin api userinfo Oauth2 ";
                $html .= " <input type=\"text\" name=\"ent[openent_oauth2_userinfo]\" value=" . $ent['openent_oauth2_userinfo'] . "><br>\n";

                break;
            default:
                $html .= "pas de parametrage possible.<br>";
        }
        // $html .= "<input name=\"ent[ent_type]\" value= \"" . $ent_list[$ent['ent_type']]['mode'] . "\" type=\"hidden\">";
        if (! $file) {
            $html .= "<br>Configuration SSO<br>Authentification OAuth2 :";
            if ($config['ent_auth'] == 1)
                $html .= " <input type=\"checkbox\" name=\"ent[ent_auth]\" value=\"1\" checked><br>\n";
            else
                $html .= " <input type=\"checkbox\" name=\"ent[ent_auth]\" value=\"1\"><br>\n";
        }
        $html .= "</div> <input type=\"submit\" value=\"Valider\">\n";
    }

    return $html;
}

/**
 * importe un fichier ENT, format csv ou xml.
 * Une archive zip est accepée, dans ce cas tous les ficihiers seront importés
 *
 * @param unknown $config
 * @return string
 */
function upload_ent($config, $files, $file_type)
{
    $html = "<br>";
    if (isset($files['csv_ent'])) {
        $html .= "Le fichier " . $files['csv_ent']['name'] . " a été chargé<br>";
        if (empty($files['csv_ent']['name'])) {
            return "<p style='color:red;'><b>ERREUR:</b> Aucun fichier n'a été; fourni !</p>\n";
        }

        $tmp_ent_file = $files['csv_ent']['tmp_name'];
        $ent_file = $files['csv_ent']['name'];
        $size_ent_file = $files['csv_ent']['size'] / 1024;

        if (empty($tmp_ent_file)) {
            $html .= "<p>L'upload du fichier <span style='color:red;'>$ent_file</span> a semble-t-il &eacute;chou&eacute;.</p>";

            $upload_max_filesize = ini_get('upload_max_filesize');
            $post_max_size = ini_get('post_max_size');

            $html .= "<p>Il se peut que le fichier fourni ait &eacute;t&eacute; trop volumineux.<br />PHP est actuellement param&eacute;tr&eacute; avec:<br />\n";
            $html .= "</p>\n";
            $html .= "<blockquote>\n";
            $html .= "<span style='color:blue;'>upload_max_filesize</span>=<span style='color:green;'>" . $upload_max_filesize . "</span><br />\n";
            $html .= "<span style='color:blue;'>post_max_size</span>=<span style='color:green;'>" . $post_max_size . "</span><br />\n";
            $html .= "</blockquote>\n";
            $html .= "<p>\n";
            $html .= "Si ces valeurs sont insuffisantes pour vos fichiers XML, il est possible de modifier les valeurs limites dans <span style='color:green;'>/etc/php/7.3/fpm/pool.d/sambaedu.conf</span>\n";
            $html .= "</p>\n";
            return $html;
        }
        $dossier_tmp_import_comptes = sys_get_temp_dir();
        // mkdir($dossier_tmp_import_comptes);
        $dest_file = $dossier_tmp_import_comptes . "/" . $ent_file;
        if (file_exists($dest_file)) {
            unlink($dest_file);
        }
        if (! is_uploaded_file($tmp_ent_file)) {
            return "<p style='color:red'>Impossible d'importer le fichier.</p><br>\n";
        }
        // $source_file = stripslashes("$tmp_ent_file");
        if (! move_uploaded_file("$tmp_ent_file", "$dest_file")) {
            return "<p style='color:red'>Impossible de copier le fichier.</p>\n";
        }
        $html .= "fichier de " . $size_ent_file . " ko correctement importé<br>\n";
        // Si jamais un fichier non dézippé est fourni
        $extension_fichier_emis = strtolower(strrchr($ent_file, "."));

        if (($extension_fichier_emis == ".zip") || ($files['csv_ent']['type'] == "application/zip")) {
            if (! extension_loaded('zip')) {
                return "Erreur, le module zip manque.";
            }

            $zip = new ZipArchive();
            if ($zip->open($dest_file) === TRUE) {
                $zip->extractTo($dossier_tmp_import_comptes);
            } else {
                return "<p style='color:red'>Echec de l'extraction de l'archive ZIP.</p>\n";
            }
            $zip->close();
            $files = scandir($dossier_tmp_import_comptes);
            $import_files = array();
            foreach ($files as $file) {
                $match = array();
                if (preg_match("/.*\.[csv|xml])$/", $file) == 1) {
                    $import_files[] = $file;
                }
            }
        } else {
            $import_files[] = $dest_file;
        }
        foreach ($import_files as $file) {
            $html .= import_ent($config, $file, $file_type);
        }
    } else {
        $html .= "Fichier d'importation des comptes : format $file_type <br>\n";
        switch ($file_type) {
            case "csv_openent":
                $html .= 'Le fichier doit être au format csv, <b>IMPERATIVEMENT encodé en UTF-8</b>, et il peut être zippé et doit obligatoirement contenir la ligne avec les intitulés des colonnes.
les champs nom, prénom, profil et classe sont obligatoires. D\'autres champs sont possibles, voir le code<p>
  Le csv format utilisé correspond à celui utilisé par l\'ENT Openent :
 <li> première ligne avec les champs :
 <li>  "externalId";"Id Siecle";"type";"lastName";"firstName";"originalLogin";"login","code";"functions";"structures";"allClasses";"childrens";"parents";"birthDate";"sex";"allGroups"
 <li>   id : identifiant unique ENT
 <li>   Id Siecle : numero eleve siecle (eleves uniquement)
 <li>   + lastName : nom en majuscule => TEST
  <li>  + firstName : prénom => Elève
  <li>  + type : Student|Teacher|Personnel|Relative|Guest
 <li>   originalLogin : login d\'origine, prenom.nom(x) x étant un nombre pour les doublons.
 <li>   login : login simplifié chois par l\'utilisateur => repris en priorité par Sambaedu
 <li>   code : mot de passe initial => repris par sambaedu si il existe
 <li>   functions : Personnel administratif
 <li>   structures : nom de(s) l\'établissemnt(s) en majuscules ,séparés par des virgules
 <li>   + allClasses : classe(s), séparées par des virgules
 <li>   allGroups : groupe(s), séparés par des virgules
 <li>   birthDate : date de naissance AAAAMMJJ
  <li>  sex : sexe M/F
     
  <p>seuls les champs avec + sont obligatoires<br>
                
<pre>
"lastName";"firstName";"type";"birthDate";"sex";"allClasses"
"TERIEUR";"Alex";"Eleve";"2000";"M";"3_E"
"TERIEUR";"Alain";"Professeur";"1969";"M";"3_E,4_G,5_A,6_F"
"BONNOT";"Jean";"Personnel";"1960";"M";"3_E"
</pre><p>
';
                break;
            case "csv_pronote":
                $html .= 'Fichier csv contenant les informations des services des professeurs. A récupérer depuis EDT en mode SPR ou proviseur, export csv/txt-> export texte, choisir professeurs, et tout laisser par défaut.
Ce fichier permet d\'associer correctement les profs à leurs classes et groupes sans devoir attendre la remontée STS ou ENT.<p><p>
Il est également possible d\'importer les élèves de cette façon, si par exemple vous ne disposez pas de fichier SIECLE.<p>
ATTENTION cet export doit plus être pensé comme complémentaire à une synchro ENT, en début d\'année par exemple, en attendant que les classes et groupes aient été remontées dans l\'ent, que comme le moyen principal de renseigner la
base profs. En effet il ne contient pas les logins, et donc ne gérera pas les doublons à l\'échelle de l\'académie ou de la région comme l\'ENT.<p>
 <li> première ligne avec les champs :
  <li>  NOM;PRENOM;FONCTION;CIVILITE;CLASSES;GROUPES
  <li>   + NOM : nom en majuscule => TEST
  <li>  + PRENOM : prénom => Elève
  <li>  + FONCTION : ENS
  <li>  EMAIL : test@ac-truc.fr 
 <li>   + CLASSES : classe(s), séparées par des virgules et CLASSE pour les élèves
 <li>   GROUPES : groupe(s), séparés par des virgules
  <li>  CIVILITE : sexe M/Mme
                    
  <p>seuls les champs avec + sont obligatoires<br>
<pre>
NOM;PRENOM;FONCTION;CIVILITE;CLASSES;GROUPES;EMAIL
TERIEUR;Alex;ENS;M;3_E,5_1;3XX,5yy,test@ac-truc.fr
</pre><p>
';
                $html .= "étapes d'export : <p>";
                $html .= "<img src=\"images/edt1.png\"><img src=\"images/edt2.png\"><p>";
                break;
            default:
                $html .= 'Le fichier doit être au format csv, <b>IMPERATIVEMENT encodé en UTF-8</b>, et il peut être zippé et doit obligatoirement contenir la ligne avec les intitulés des colonnes.
les champs nom, prénom, profil et classe sont obligatoires. D\'autres champs sont possibles, voir le code<p>
  Le csv format utilisé correspond à celui utilisé par l\'ENT Openent :
 <li> première ligne avec les champs :
  <li>  "externalId";"Id Siecle";"type";"lastName";"firstName";"originalLogin";"login","code";"functions";"structures";"allClasses";"childrens";"parents";"birthDate";"sex";"allGroups"
 <li>   id : identifiant unique ENT
 <li>   Id Siecle : numero eleve siecle (eleves uniquement)
 <li>   + lastName : nom en majuscule => TEST
  <li>  + firstName : prénom => Elève
  <li>  + type : Student|Teacher|Personnel|Relative|Guest
 <li>   originalLogin : login d\'origine, prenom.nom(x) x étant un nombre pour les doublons.
 <li>   login : login simplifié chois par l\'utilisateur => repris en priorité par Sambaedu
 <li>   code : mot de passe initial => repris par sambaedu si il existe
 <li>   functions : Personnel administratif
 <li>   structures : nom de(s) l\'établissemnt(s) en majuscules ,séparés par des virgules
 <li>   + allClasses : classe(s), séparées par des virgules
 <li>   allGroups : groupe(s), séparés par des virgules
 <li>   birthDate : date de naissance AAAAMMJJ
  <li>  sex : sexe M/F
                    
  <p>seuls les champs avec + sont obligatoires<br>
                    
<pre>
"lastName";"firstName";"type";"birthDate";"sex";"allClasses"
"TERIEUR";"Alex";"Student";"20000120";"M";"3_E"
"TERIEUR";"Alain";"Teacher";"19690528";"M";"3_E,4_G,5_A,6_F"
"BONNOT";"Jean";"Personnel";"19600131";"M";"3_E"
</pre><p>
';
        }
        $html .= "<input title=\"chemin du fichier csv\" size=\"100\" name=\"csv_ent\" type=\"file\"></input><p>\n";
    }
    return $html;
}

function form_sync_ent($config)
{
    $uploaddir = "/tmp/";
    $nomFichier = basename($_FILES['ldiffile']['name']);
    if ($nomFichier != '') {
        $ldiffile = $uploaddir . $nomFichier;
        echo "<h3>Transfert du fichier ldif</h3>\n";
        if (move_uploaded_file($_FILES['ldiffile']['tmp_name'], $ldiffile)) {
            echo "Le fichier '$nomFichier' a &#233;t&#233; transf&#233;r&#233; avec succ&#232;s.<br>\n";
            echo "<H3>" . gettext("Publication du fichier") . "</H3>";
            echo "<PRE>\n";

            system("ldapadd -x -c -h $ldap_server -D $ldap_admin_name,$ldap_base_dn -w $ldap_admin_passwd -f $ldiffile");
            echo "</PRE>\n";
            unlink("$ldiffile");
        } else {
            echo "Erreur de transfert du fichier '$nomFichier'.<br>\n";
        }
    }
}

function form_map_users($config, $global = false)
{
    $ent = apcu_fetch('new');
    $ad = ldap_users_group($config, apcu_fetch('ad'));

    $html = "<TABLE border=1 width=\"80%\">\n";
    $html .= "<th colspan=4 class=menuheader style=\"height: 30\">Mise en correspondance des utilisateurs</th>\n";

    $html .= "<tr><td class='menuheader' align=\"center\">Type</td>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant ENT</td>";
    $html .= "<td class='menuheader' align=\"center\">Classe(s)</td>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant AD</td>";
    $html .= "</tr>\n";

    foreach ($ent as $index => $user) {
        if ($user['type'] == "Teacher") {
            $user['type'] = "prof";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($ad['prof'] as $prof) {
                $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ")</option>";
            }
            if ($global) {
                foreach ($ad['prof'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", sans classe)</option>";
                }
                foreach ($ad['trash'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Corbeille)</option>";
                }
                foreach ($ad['administratif'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Administratif)</option>";
                }
            }

            $html .= "</select></td>";
        } elseif ($user['type'] == "Student") {
            $user['type'] = "eleve";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($user['allClasses'] as $classe) {
                foreach ($ad['eleve'][$classe['name']] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ")</option>";
                }
            }
            if ($global) {
                foreach ($ad['eleve']['sans'] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ", sans classe)</option>";
                }
                foreach ($ad['trash'] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ", corbeille)</option>";
                }
            }

            $html .= "</select></td>";
        }
        if ($user['type'] == "Personnel" || $user['type'] == "Guest") {
            $user['type'] = "administratif";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($ad['administratif'] as $prof) {
                $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ")</option>";
            }
            if ($global) {
                foreach ($ad['prof'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Administratif)</option>";
                }
                foreach ($ad['trash'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", corbeille)</option>";
                }
            }

            $html .= "</select></td>";
        } else
            $type = "";
        $html .= "</tr>";
        $html .= "<input type=\"hidden\" name=\"form\" value=\"match\">";
    }

    $html .= "</table>";
    return $html;
}

function form_rename_users($config, $global = false)
{
    $match = apcu_fetch('match');

    $html = "<TABLE border=1 width=\"80%\">\n";
    $html .= "<th colspan=4 class=menuheader style=\"height: 30\">Renommage des utilisateurs</th>\n";

    $html .= "<tr>";
    $html .= "<td class='menuheader' align=\"center\">type</td>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant commun</td>";
    $html .= "<td class='menuheader' align=\"center\">Classe(s)</td>";
    $html .= "<td class='menuheader' align=\"center\">Login proposé</td>";
    $html .= "<td class='menuheader' align=\"center\">Renommer</td>";
    $html .= "</tr>\n";

    foreach ($match as $index => $user) {
        if (($user['ent']['login'] != $user['ad']['cn']) || ($user['ent']['login'] != mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName']) && $user['ent']['login'] != $user['ent']['originalLogin'])) {
            if ($user['ent']['type'] == "Teacher") {
                $user['ent']['type'] = "prof";
            } elseif ($user['ent']['type'] == "Student") {
                $user['ent']['type'] = "eleve";
            } elseif ($user['ent']['type'] == "Personnel") {
                $user['ent']['type'] = "administratif";
            }

            $html .= "<tr><td>" . $user['ent']['type'] . "</td>";
            $html .= "<td>" . $user['ent']['firstName'] . " " . $user['ent']['lastName'] . " (" . $user['ent']['login'] . ")</td>";
            if (count($user['ent']['allClasses']) > 0 && ! empty($user['ent']['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['ent']['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>" . mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName']) . "</td>\n";
            $html .= "<td>";
            $html .= "<input type=\"checkbox\" name=\"ad[" . $user['ent']['login'] . "]\" ";
            if ($global) {
                $html .= "checked";
            }
            $html .= "></td>";
            $html .= "</tr>";
        }
    }
    $html .= "<input type=\"hidden\" name=\"form\" value=\"rename\">";
    $html .= "</table>";
    return $html;
}

function form_trash_users($config, $global = false)
{
    $ad = ldap_users_group($config, apcu_fetch('ad'));

    $html = "<TABLE border=1 width=\"80%\">\n";
    $html .= "<th colspan=5 class=menuheader style=\"height: 30\">Mise à la corbeille des utilisateurs</th>\n";

    $html .= "<tr>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant AD</td>";
    $html .= "<td class='menuheader' align=\"center\">Nom</td>";
    $html .= "<td class='menuheader' align=\"center\">type</td>";
    $html .= "<td class='menuheader' align=\"center\">Classe(s)</td>";
    $html .= "<td class='menuheader' align=\"center\">Supprimer</td>";
    $html .= "</tr>\n";

    function display_trash_user(&$html, $user, $classe, $type, $selected)
    {
        if (! $user['protected'] && $type != "trash") {
            $html .= "<tr><td>" . $user['cn'] . "</td>";
            $html .= "<td>" . $user['prenom'] . " " . $user['nom'] . " </td>";
            $html .= "<td>" . $type . "</td>";
            $html .= "<td>" . $classe . "</td>\n";
            $html .= "<td>";
            $html .= "<input type=\"checkbox\" name=\"ad[" . $user['cn'] . "]\" ";
            if ($selected || $user['trash']) {
                $html .= "checked";
            }
            $html .= "></td>";
            $html .= "</tr>";
        }
    }
    foreach ($ad as $type => $groupes) {
        if ($type == "eleve") {
            foreach ($groupes as $classe => $users) {
                foreach ($users as $user) {
                    display_trash_user($html, $user, $classe, $type, $global);
                }
            }
        } else {
            foreach ($groupes as $user) {
                display_trash_user($html, $user, "", $type, $global);
            }
        }
    }

    $html .= "<input type=\"hidden\" name=\"form\" value=\"trash\">";
    $html .= "</table>";
    return $html;
}

function form_ent_next($type = "apply")
{
    $types = array(
        "nouveau" => "1 - Récupération des données",
        "nouveau_force" => "1 - Forcer la récupération",
        "read_file" => "1 - Analyse du fichier des données",
        // "merge_pronote" => "1 - Fusion des données prof de Pronote",
        "match_groups" => "2 - Correspondances par groupe",
        "match_global" => "3 - Correspondances globales",
        "rename" => "4 - renommer des utilisateurs",
        "rename_global" => "5 - renommer tous les utilisateurs",
        "trash" => "6 - Mettre des utilisateurs à la corbeille",
        "trash_all" => "6 - Mettre tous les utilisateurs à la corbeille",
        "apply" => "7 - Valider",
        "commit" => "8 - Appliquer les changements"
    );
    $html = "Selectionner l'étape suivante :  <SELECT NAME=\"action\"\">";
    foreach ($types as $t => $txt) {
        $html .= "<option value=\"$t\"";
        if ($type == $t) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "<input type=\"submit\" value=\"Valider\">";
    $html .= "</SELECT><br>";
    return $html;
}

?>
