<?php

/** fonctions pour verrouiller un poste lors d'un état transitoire, par exemple lors de la connexion rdp
 * entre le momentoù le poste est choisi et le moment où la session est effectivement ouverte, il ne faut pas 
 * que d'autres utilisateurs puissent le choisir.
 * 
 * @param unknown $computer
 * @param array $lock
 * @return unknown|boolean
 */
function computer_is_locked($computer, $lock = [])
{
    if (empty($lock)) {
        $lock = apcu_fetch('computer_lock');
    }
    if (isset($lock[$computer])) {
        return $lock[$computer];
    } else {
        return false;
    }
}

function computer_lock($computer, $user, &$lock = [], $store = true)
{
    if (empty($lock)) {
        $lock = apcu_fetch('computer_lock');
    }
    $lock[$computer] = $user;
    if ($store) {
        apcu_store('computer_lock', $lock, 600);
    }
}

function computer_unlock($computer, &$lock = [], $store = true)
{
    if (empty($lock)) {
        $lock = apcu_fetch('computer_lock');
    }
    if (! empty($lock[$computer])) {
        unset($lock[$computer]);
    }
    if ($store) {
        apcu_store('computer_lock', $lock, 600);
    }
}

/** enregistrement des logs de demarrage ou de connexion
 * @param unknown $config
 * @param unknown $user
 * @param unknown $machine
 * @param unknown $os
 * @param unknown $action
 */
function log_connexion($config, $user, $machine, $os, $action)
{
    $machine = strtolower($machine);
    // statut de la machine dans la base sql.
    $status = get_machine_status($config, $machine);
    if ($action == "repair_startup" || $action == "repair_logon") {
        $error = $status['error'] + 1;
    } else {
        $error = 0;
    }
    $db_link = db_connect($config);

    switch ($action) {
        case 'repair_startup':
        case 'startup':

            // on considère qu'au bout de 3 minutes, l'action aurait dû se terminer.
            // le score permet de classer les machines qui bootent vite !
            $score = 300 - (time() - strtotime($status['starttime']));
            if ($score <= 0) {
                $score = 0;
            }
            switch ($status['state']) {
                case "off":
                    // nouveau boot
                    $query = mysqli_prepare($db_link, "insert into `machines` (name, os, starttime, error) values (?, ?, now(), ?)");
                    mysqli_stmt_bind_param($query, "ssi", $machine, $os, $error);
                    break;
                case "wol":
                    $query = mysqli_prepare($db_link, "update `machines` set wol=?, os=?, error=?, starttime=now()  where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "isis", $score, $os, $error, $machine);
                    break;
                case "ipxe":
                    $query = mysqli_prepare($db_link, "update `machines` set ipxe=?, starttime=now(), os=?, error=? where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "isis", $score, $os, $error, $machine);
                    break;
                default:
                    // if ($status['state'] != $os) {
                    $query = mysqli_prepare($db_link, "update `machines` set starttime=now(), os=?, error=? where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "sis", $os, $error, $machine);
                // }
            }
            mysqli_stmt_execute($query);
            mysqli_stmt_close($query);

            $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name=? and logouttime=0 and (DATE_SUB(now(),INTERVAL 5 MINUTE) >= logintime)");
            mysqli_stmt_bind_param($query, "s", $machine);
            // computer_unlock($machine);
            break;
        case 'shutdown':
            $query = mysqli_prepare($db_link, "update `machines` set stoptime=now() where name=? and stoptime=0");
            mysqli_stmt_bind_param($query, "s", $machine);
            mysqli_stmt_execute($query);
            mysqli_stmt_close($query);
            $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name=? and logouttime=0");
            mysqli_stmt_bind_param($query, "s", $machine);
            break;
        case 'repair_logon':
        case 'logon':
            if (! empty($user) && $user != "root" && $user != "Debian-gdm" && $user != "lightdm") {
                if ($status['state'] != $os) {
                    $query = mysqli_prepare($db_link, "update `machines` set os=?, error=? where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "sis", $os, $error, $machine);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);
                }
                $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where username=? and netbios_name=? and logouttime=0");
                mysqli_stmt_bind_param($query, "ss", $user, $machine);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);

                $query = mysqli_prepare($db_link, "insert into `connexions` (username, netbios_name, logintime) values (?, ?, now())");
                mysqli_stmt_bind_param($query, "ss", $user, $machine);
            }
            computer_unlock($machine);
            break;
        case 'logoff':
            if (! empty($user) && $user != "root" && $user != "Debian-gdm" && $user != "lightdm") {
                $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where username=? and netbios_name=? and logouttime=0");
                mysqli_stmt_bind_param($query, "ss", $user, $machine);
            }
            break;
        case 'blacklist':
            $query = mysqli_prepare($db_link, "update `machines` set wol=0, ipxe=0, stoptime=now(), os=?  where name=? and stoptime=0");
            mysqli_stmt_bind_param($query, "ss", $os, $machine);
            break;
    }
    if (isset($query) && $query) {
        mysqli_stmt_execute($query);
        mysqli_stmt_close($query);
    }
    mysqli_close($db_link);
}

function get_machine_status($config, $machine)
{
    $machine = strtolower($machine);

    $db_link = db_connect($config);
    // $query = mysqli_prepare($db_link, "select starttime, os from `machines` where name = ? and stoptime=0 order by id desc limit 1");
    $query = mysqli_prepare($db_link, "select starttime, stoptime, os, error from `machines` where name = ? order by id desc limit 1");
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_starttime, $res_stoptime, $res_os, $res_error);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    $query2 = mysqli_prepare($db_link, "select username, logintime from `connexions` where netbios_name = ? and logouttime=0");
    mysqli_stmt_bind_param($query2, "s", $machine);
    mysqli_stmt_execute($query2);
    mysqli_stmt_bind_result($query2, $res_username, $res_logintime);
    mysqli_stmt_store_result($query2);
    $num_rows2 = mysqli_stmt_num_rows($query2);
    if ($num_rows2 > 0) {
        while (mysqli_stmt_fetch($query2)) {
            $tab['user'][] = array(
                'name' => $res_username,
                'logintime' => $res_logintime
            );
        }
    }
    $tab['state'] = "off";
    $tab['error'] = 0;
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab['state'] = $res_os;
            $tab['error'] = $res_error;
            if ($res_stoptime == 0) {
                $tab['starttime'] = $res_starttime;
            } elseif (isset($tab['user']) && count($tab['user']) > 0) {
                // on ne devrait jamais arriver ici...
                $query3 = mysqli_prepare($db_link, "insert into `machines` (name, os, starttime, error) values (?, ?, ?, now())");
                mysqli_stmt_bind_param($query3, "ssi", $machine, $res_os, $res_error);
                mysqli_stmt_execute($query3);
                mysqli_stmt_close($query3);
                $tab['starttime'] = time();
            } else {
                $tab['state'] = "off";
                $tab['starttime'] = 0;
            }
        }
    }
    mysqli_stmt_close($query2);
    mysqli_stmt_close($query);

    $query2 = mysqli_prepare($db_link, "select wol, ipxe from `machines` where name = ? and stoptime!=0 order by id desc limit 1");
    mysqli_stmt_bind_param($query2, "s", $machine);
    mysqli_stmt_execute($query2);
    mysqli_stmt_bind_result($query2, $res_wol, $res_ipxe);
    mysqli_stmt_store_result($query2);
    $num_rows2 = mysqli_stmt_num_rows($query2);
    if ($num_rows2 > 0) {
        while (mysqli_stmt_fetch($query2)) {
            $tab['score'] = $res_wol + $res_ipxe;
        }
    } else
        $tab['score'] = 0;

    mysqli_close($db_link);
    $tab['locked']  = computer_is_locked($machine);
    return $tab;
}

function get_user_status($config, $user)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select netbios_name, logintime from `connexions` where username=? and logouttime=0 order by logintime desc");
    mysqli_stmt_bind_param($query, "s", $user);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_machine, $res_logintime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'logintime' => $res_logintime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_user_connexions($config, $user, $start = 0, $nb = 10)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select netbios_name, logintime, logouttime from `connexions` where username=? order by id desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $user);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_machine, $res_logintime, $res_logouttime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'logintime' => $res_logintime,
                'logouttime' => $res_logouttime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_machine_connexions($config, $machine, $start = 0, $nb = 10)
{
    $machine = strtolower($machine);

    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select username, logintime, logouttime from `connexions` where netbios_name=? order by id desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_user, $res_logintime, $res_logouttime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'user' => $res_user,
                'logintime' => $res_logintime,
                'logouttime' => $res_logouttime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_machine_startups($config, $machine, $start = 0, $nb = 10)
{
    $machine = strtolower($machine);

    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select os, starttime, stoptime, error from `machines` where name=? order by starttime desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_os, $res_starttime, $res_stoptime, $res_error);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'os' => $res_os,
                'starttime' => $res_starttime,
                'stoptime' => $res_stoptime,
                'error' => $res_error
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function diagnostic_machine($config, $machine)
{
    $machine = strtolower($machine);

    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select os, ipxe, wol, starttime, stoptime from `machines` where name=? and error=1 order by starttime desc limit 5");
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_os, $res_starttime, $res_stoptime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'os' => $res_os,
                'starttime' => $res_starttime,
                'stoptime' => $res_stoptime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function open_sessions($config)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select username, netbios_name, logintime from `connexions` where logouttime=0 order by username");
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_username, $res_machine, $res_logintime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'user' => $res_username,
                'logintime' => $res_logintime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function clean_machines_status($config)
{

    // on cherche les machines demarrées win ou linux ou depuis plus de 2 minutes
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select name, wol, ipxe, os, error from `machines` where stoptime=0 and
                    (DATE_SUB(now(),INTERVAL 2 MINUTE) >= starttime) order by id desc");
    // (os='windows' or os='linux' or DATE_SUB(now(),INTERVAL 2 MINUTE) >= starttime) order by id desc");
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_name, $res_wol, $res_ipxe, $res_os, $res_error);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    $out = "";
    if ($num_rows > 0) {
        // $out = "nettoyage des connexions des machines plantees :<br>\n";
        while (mysqli_stmt_fetch($query)) {
            if (! $ping_os = fping($res_name)) {
                // pas de ping, on ferme les cnx
                if ($res_os == "wol") {
                    $out .= strtolower($res_name) . " n'a pas été allumée, soit le WOL ne fonctionne pas, soit la machine est débranchée ou n'existe plus<br>\n";
                    $res_wol = 0;
                } elseif ($res_os == "ipxe") {
                    $out .= strtolower($res_name) . " a booté en ipxe, mais n'a pas démarré correctement de système d'exploitation ensuite<br>\n";
                    $res_ipxe = 0;
                } else {
                    $out .= "Détection de l'extinction de " . strtolower($res_name) . " os : " . $res_os . " : Soit la gpo logs ne s'applique pas, soit la machine a été arrêtée brutalement.<br>\n";
                    $res_os = "stop";
                }
                $res_error ++;
                $query2 = mysqli_prepare($db_link, "update `machines` set wol=?, ipxe=?, os=?, error=?, stoptime=now() where name='" . $res_name . "' and stoptime=0");
                mysqli_stmt_bind_param($query2, "iisi", $res_wol, $res_ipxe, $res_os, $res_error);
                mysqli_stmt_execute($query2);
                mysqli_stmt_close($query2);
                $query2 = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name='" . $res_name . "' and logouttime=0");
                mysqli_stmt_execute($query2);
                mysqli_stmt_close($query2);
            } else {
                if ($res_os != $ping_os) {
                    $out .= "Etat d'allumage actualisé pour " . strtolower($res_name) . " : la gpo logs ou les scripts de démarrage Linux ne s'appliquent pas<br>\n";
                    if ($ping_os == "windows" && $res_error > 4) {
                        $out .= "tentative de réparation des gpo pour " . strtolower($res_name) . "...<br>";
                        $out .= exec("smbclient -k //" . $res_name . "/c\$ -c " . escapeshellarg("deltree \\windows\\system32\\grouppolicy;deltree \\windows\\system32\\grouppolicyusers") . " 2>/dev/null");
                        $out .= "<br>";
                        log_connexion($config, "", $res_name, $ping_os, "startup");
                    } else {
                        log_connexion($config, "", $res_name, $ping_os, "repair_startup");
                    }
                }
                $smb_status = machine_smbstatus($config, $res_name);
                if (! empty($smb_status && $smb_status != "wpkg")) {
                    $sql_status = get_machine_status($config, $res_name);
                    if (! @in_array($smb_status, array_column($sql_status['user'], 'name'))) {
                        log_connexion($config, $smb_status, $res_name, $ping_os, "repair_logon");
                        $out .= "connexion actualisée pour " . strtolower($res_name) . " utilisateur " . $smb_status . "<br>\n";
                    }
                }
            }
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $out;
}
?>