<?php

function make_lockscreen($config, $machine, $format)
{
    global $margin, $fontSize;

    if (! empty($machine['dn'])) {
        $salle = ldap_dn2cn(ldap_dn2oudn($machine['dn']));
    } else {
        $salle = "";
    }
    $texte = $machine['cn'] . " (" . $salle . ")";
    // image par défaut
    $image = "/usr/share/sambaedu/applications/wallpaper/default.jpg";
    $files = scandir("/etc/sambaedu/applications/wallpaper");
    // image pour la salle
    if (in_array("lockscreen-$salle.jpg", $files)) {
        $image = "/etc/sambaedu/applications/wallpaper/lockscreen-$salle.jpg";
    } elseif (in_array("lockscreen.jpg", $files)) {
        // image lockscreen par défaut pour l'établissement
        $image = "/etc/sambaedu/applications/wallpaper/lockscreen.jpg";
    } elseif (in_array("default.jpg", $files)) {
        // image par défaut pour l'établissement
        $image = "/etc/sambaedu/applications/wallpaper/default.jpg";
    }
    $imagick = new Imagick(realpath($image));

    $draw = new ImagickDraw();
    $draw->setStrokeColor("black");
    $draw->setFillColor("white");

    $draw->setStrokeWidth(1);
    $draw->setFontSize($fontSize);

    $draw->setGravity(Imagick::GRAVITY_NORTHEAST);
    $draw->setFont(realpath("/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf"));
    $imagick->annotateimage($draw, $margin + 160, $margin, 0, $texte);

    $logo = new Imagick("/usr/share/sambaedu/applications/wallpaper/sambaedu.png");

    $imagick->compositeImage($logo, Imagick::COMPOSITE_OVER, $margin + 160, $margin);

    if (file_exists($etab = "/etc/sambaedu/applications/wallpaper/logo.png")) {

        $logo = new Imagick($etab);
        $logoWidth = $logo->getImageWidth();
        $logoHeight = $logo->getImageHeight();
        $imagick->compositeImage($logo, Imagick::COMPOSITE_OVER, 1920 - $margin - 160 - $logoWidth, 1080 - $margin - $logoHeight);
    }
    $imagick->setImageFormat($format);
    if ($format == "png") {
        $imagick->resizeimage(1280, 720, Imagick::FILTER_QUADRATIC, 1);
        $imagick->setOption('png:compression-level', 8);
    } else {
        $imagick->setCompressionQuality(20);
    }
    return $imagick->getImageBlob();
}

function make_wallpaper($config, $machine, $user, $format)
{
    global $margin, $fontSize;
    
    if (! empty($machine['dn'])) {
        $salle = ldap_dn2cn(ldap_dn2oudn($machine['dn']));
    } else {
        $salle = "";
    }
    $groups = [];
    if (isset($user['cn'])) {
        $groups[] = $user['cn'];
        foreach ($user['memberof'] as $group) {
            $groups[] = ldap_dn2cn($group);
        }

        $groups[] = $user['cn'];
        $full = false;
        $t = "";
        $map = [
            "/home" => "Espace perso",
            "/var/sambaedu" => "Espace Classe"
        ];
        $quotas = repquota();
        $keys = array_keys(array_column($quotas, "nom"), $user['cn']);
        foreach ($keys as $key) {
            $quota = $quotas[$key];
            if ($quota['overfill']) {
                $full = true;
                $t .= $map[$quota['partition']] . " : utilisé  : " . round($quota['utilise'] / 1024, 0) . " Mo, limite : " . round($quota['quotasoft'] / 1024, 0) . " Mo temps restant : " . $quota['grace'] . " jours\n";
            }
        }
        if ($full) {
            $imagick = new Imagick();
            $texte = $user['cn'] . " : Espace de stockage personnel saturé,\nveuillez libérer de l'espace et vous reconnecter pour retrouver un fonctionement normal !\n";
            $texte .= $t;
            $imagick->newImage(1920, 1080, new ImagickPixel('red'));
        } else {
            // image par défaut
            $image = "/usr/share/sambaedu/applications/wallpaper/default.jpg";
            $files = scandir("/etc/sambaedu/applications/wallpaper");
            // image par défaut de l'établissement
            if (in_array("default.jpg", $files)) {
                $image = "/etc/sambaedu/applications/wallpaper/default.jpg";
            } elseif (in_array("wallpaper.jpg", $files)) {
                // image wallpaper par défaut de l'établissemnt
                $image = "/etc/sambaedu/applications/wallpaper/wallpaper.jpg";
            }
            // on cherche si le groupe principal a une image
            foreach ([
                "Profs",
                "Eleves",
                "Administratifs"
            ] as $type) {
                if (in_array($type, $groups)) {
                    if ($key = array_search("$type.jpg", $files)) {
                        $image = "/etc/sambaedu/applications/wallpaper/$type.jpg";
                        unset($groups[$key]);
                    } elseif (in_array("wallpaper.jpg", $files)) {
                        $image = "/etc/sambaedu/applications/wallpaper/wallpaper.jpg";
                    }
                    break;
                }
            }
            // on cherche si les autres groupes ont une image, le dernier gagne
            foreach ($groups as $group) {
                if (in_array("$group.jpg", $files)) {
                    $image = "/etc/sambaedu/applications/wallpaper/$group.jpg";
                }
            }
            // si une image perso est définie on l'impose
            if (! empty($config['perso_wallpaper']) && file_exists("/home/" . $user['cn'] . "/Photos/wallpaper.jpg")) {
                $image = "/home/" . $user['cn'] . "/Photos/wallpaper.jpg";
            }
            $fullname = $user['fullname'] ?? $user['cn'];
            $texte = "Utilisateur : " . $fullname . " (" . $type . "), machine : " . $machine['cn'] . " (" . $salle . ")";
            if (file_exists($image)) {
                $imagick = new Imagick(realpath($image));
                //$imagick->resizeimage(1920, 1080, Imagick::FILTER_QUADRATIC, 1);
            } else {
                $imagick = new Imagick();
                $texte .= "probleme pour trouver $image";
                $imagick->newImage(1920, 1080, new ImagickPixel('red'));
            }
        }
    } else {
        $image = "/usr/share/sambaedu/applications/wallpaper/default.jpg";
        $imagick = new Imagick(realpath($image));
        $texte = "Utilisateur inconnu, machine : " . $machine['cn'] . " (" . $salle . ")";
    }
    $draw = new \ImagickDraw();
    $draw->setStrokeColor("black");
    $draw->setFillColor("white");

    $draw->setStrokeWidth(1);
    $draw->setFontSize($fontSize);
    $draw->setGravity(Imagick::GRAVITY_NORTHEAST);

    $draw->setFont(realpath("/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf"));
    $imagick->annotateimage($draw, $margin, $margin, 0, $texte);

    // $logo = new Imagick("/usr/share/sambaedu/applications/wallpaper/sambaedu.png");
    // $imagick->compositeImage($logo, Imagick::COMPOSITE_OVER, 960, 800);
    $imagick->setImageFormat($format);
    return $imagick->getImageBlob();
}

function make_icon($path)
{
    if (! is_file($path)){
        if ( is_file("/etc/sambaedu/applications/wallpaper/default.jpg")){
            $path = "/etc/sambaedu/applications/wallpaper/default.jpg";
        } else {
        $path = "/usr/share/sambaedu/applications/wallpaper/default.jpg";
        }
    }
    $imagick = new Imagick($path);
    $imagick->scaleImage(0, 100);
    $imagick->setImageFormat("png");
    return $imagick->getImageBlob();
}

function upload_wallpaper($config, $path)
{
    $html = "";
    $logo = $_POST['logo'] ?? 0;
    if (isset($_POST['delete'][$path])) {
        unlink($path);
    } elseif (isset($_FILES['image']['name'][$path])) {
        $tmp_image_file = $_FILES['image']['tmp_name'][$path] ?? "";
        $image_file = $_FILES['image']['name'][$path] ?? "";
        if (empty($tmp_image_file)) {
            $html .= "<p>L'upload du fichier <span style='color:red;'>$image_file</span> a semble-t-il &eacute;chou&eacute;.</p>";

            $upload_max_filesize = ini_get('upload_max_filesize');
            $post_max_size = ini_get('post_max_size');

            $html .= "<p>Il se peut que le fichier fourni ait &eacute;t&eacute; trop volumineux.<br />PHP est actuellement param&eacute;tr&eacute; avec:<br />\n";
            $html .= "</p>\n";
            $html .= "<blockquote>\n";
            $html .= "<span style='color:blue;'>upload_max_filesize</span>=<span style='color:green;'>" . $upload_max_filesize . "</span><br />\n";
            $html .= "<span style='color:blue;'>post_max_size</span>=<span style='color:green;'>" . $post_max_size . "</span><br />\n";
            $html .= "</blockquote>\n";
            $html .= "<p>\n";
            $html .= "Si ces valeurs sont insuffisantes pour vos fichiers, il est possible de modifier les valeurs limites dans <span style='color:green;'>/etc/php/7.3/fpm/pool.d/sambaedu.conf</span>\n";
            $html .= "</p>\n";
        }
        $dossier_tmp_import_comptes = sys_get_temp_dir();
        // mkdir($dossier_tmp_import_comptes);
        $dest_file = $dossier_tmp_import_comptes . "/" . $image_file;
        if (file_exists($dest_file)) {
            unlink($dest_file);
        }
        if (! is_uploaded_file($tmp_image_file)) {
            $html .= "<p style='color:red'>Impossible d'importer le fichier.</p><br>\n";
        }
        // $source_file = stripslashes("$tmp_image_file");
        if (! move_uploaded_file("$tmp_image_file", "$dest_file")) {
            $html .= "<p style='color:red'>Impossible de copier le fichier.</p>\n";
        }
        // $html .= "fichier de " . $size_image_file . " ko correctement importé<br>\n";
        if (! $imagick = new Imagick($dest_file)) {
            $html .= "<p style='color:red'>Format d'image invalide.</p><br>\n";
        } else {
            if ($logo == 1) {
                $imagick->scaleImage(0, 300);
                $imagick->setImageFormat('png');
                $imagick->writeImage($path);
            } else {
                $imagick->resizeimage(1920, 1080, Imagick::FILTER_QUADRATIC, 1);
                $imagick->setImageFormat('jpg');
                $imagick->writeImage($path);
            }
        }
        // $html .= "L'image a correctectement été importée";
    }
    if (isset($_POST['modif'][$path])) {
        switch ($path) {
            case "/etc/sambaedu/applications/wallpaper/default.jpg":
                $html .= "Image par défaut : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                break;
            case "/etc/sambaedu/applications/wallpaper/logo.png":
                $html .= "Logo de l'établissement (fond transparent) : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                $html .= "<input type=hidden name=logo value=1>";
                break;
            case "/etc/sambaedu/applications/wallpaper/wallpaper.jpg":
                $html .= "Fond d'écran par défaut : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                break;
            case "/etc/sambaedu/applications/wallpaper/lockscreen.jpg":
                $html .= "Ecran de login par défaut : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                break;
            default:
                $html .= "Fond d'écran pour $path : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
        }
        $html .= "Choisir une image<p>";
        $html .= "<input title=\"Fichier image\" size=\"50\" name=\"image[" . $path . "]\" type=\"file\" onchange=submit()></input>\n";
        $html .= "<input type=submit name=delete[" . $path . "] value=Supprimer><p>";
    } else {

        switch ($path) {
            case "/etc/sambaedu/applications/wallpaper/default.jpg":
                $html .= "Image par défaut : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                break;
            case "/etc/sambaedu/applications/wallpaper/logo.png":
                $html .= "Logo de l'établissement : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                break;
            case "/etc/sambaedu/applications/wallpaper/wallpaper.jpg":
                $html .= "Fond d'écran par défaut : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                break;
            case "/etc/sambaedu/applications/wallpaper/lockscreen.jpg":
                $html .= "Ecran de login par défaut : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
                break;
            default:
                $html .= "Fond d'écran pour " . explode(".", explode("/", $path)[5])[0] . " : <img src=\"../gpo/wallpaper_out.php?action=icone&user=" . urlencode($path) . "\">";
        }
        $html .= "<input type=submit name=modif[" . $path . "] value=Modifier...><p>";
    }
    // unset($_FILES);
    // unset($_POST);
    return $html;
}
?>
